/**

        project     PAFEE
 *      @file:     pafee_stdlib.c
 *      @author:   Technocrats
 *      @brief:    Structure tool define header

*/
#ifdef PAFEE_LIB_STDLIB_ENABLE

//#include <stdio.h>
#include <limits.h>
#include "pafee_stdlib.h"
/*********************************************************/
/*                                                       */
/* Static function prototype                             */
/*                                                       */
/*********************************************************/
/**
 * @brief  Check character is digit or not(isdigit)
 * @param  checkc character
 * @return result
 * @retval is digit  result=1
 * @retval not digit result=0
 */
static int CheckDigit(
	const char character
);

/**
 * @brief  Check character is digit or not(isdigit)
 * @param  checkc character
 * @return result
 * @retval is digit or alpha  result=num
 * @retval not digit or alpha result=-1
 */
static int CheckAlpha(
	const char character
);

/**
 * @brief  get string 
 * @param  convert string
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int GetSignstring(
	const char *pString,
	const char **endPointer
);

/*********************************************************/
/*                                                       */
/* Global function implementation                        */
/*                                                       */
/*********************************************************/
/**
 * @brief  ascii to integer
 * @param  convert string
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
int pafee_atoi(
	const char *convertString
)
{
	char *pNotSpace;
	int sign;
	int result;

//	printf("pafee_");
	sign = GetSignstring(convertString, &pNotSpace);

	result = 0;
	while((*pNotSpace >= '0') && (*pNotSpace <= '9'))
	{
		result = result * 10 + *pNotSpace - '0';

		pNotSpace++;
	}

	if (sign != 0)
		result = -result;

	return result;
}

/**
 * @brief  ascii to long integer
 * @param  convert string
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
long pafee_atol(
	const char *convertString
)
{
	char *pNotSpace;
	int sign;
	long result;

//	printf("pafee_");
	sign = GetSignstring(convertString, &pNotSpace);

	result = 0;
	while((*pNotSpace >= '0') && (*pNotSpace <= '9'))
	{
		result = result * 10 + *pNotSpace - '0';

		pNotSpace++;
	}

	if (sign != 0)
		result = -result;

	return result;
}

/**
 * @brief  string to long integer
 * @param  convert string
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
long pafee_strtol(
	const char * pString,
	char ** pEnd,
	int base
)
{
	long result;
	int sign;
	int number;
	char *character;

//	printf("pafee_");
	sign = GetSignstring(pString, &character);
	if(character[0] == '0')
	{
		character++;
		if((character[0] == 'x') || (character[0] == 'X'))
		{
			if((base == 0) || (base == 16))
			{
				character++;
				base = 16;
			}
		}
		else if(base == 0)
		{
			base = 8;
		}
	}
	else if(base == 0)
	{
		base = 10;
	}

	result = 0;
	number = CheckAlpha(*character);
	while(number != -1)
	{
		if (number >= base)
		{
			break;
		}

		if ((result > ((LONG_MAX - number - sign) / base)) || (result < 0))
		{
//			errno = ERANGE;
			if(sign == -1)
			{
				result = LONG_MIN;
			}
			else
			{
				result = LONG_MAX;
			}
		}
		else
		{
			result = result * base + number;
		}

		character++;
		number = CheckAlpha(*character);
	}

	if (pEnd != 0)
	{
		*pEnd = (char*)character;
	}

	if (sign != 0)
	{
		result = -result;
	}

	return result;
}

/**
 * @brief  string to unsigned long integer
 * @param  convert string
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
unsigned long pafee_strtoul(
	const char * pString,
	char ** pEnd,
	int base
)
{
	unsigned long result;
	int sign;
	int number;
	char *character;

//	printf("pafee_");
	sign = GetSignstring(pString, &character);

	if(character[0] == '0')
	{
		character++;
		if((character[0] == 'x') || (character[0] == 'X'))
		{
			if((base == 0) || (base == 16))
			{
				character++;
				base = 16;
			}
		}
		else if(base == 0)
		{
			base = 8;
		}
	}
	else if(base == 0)
	{
		base = 10;
	}

	result = 0;
	number = CheckAlpha(*character);
	while(number != -1)
	{
		if (number >= base)
		{
			break;
		}

		if (result > ((ULONG_MAX - number) / base))
		{
//			errno = ERANGE;
			result = ULONG_MAX;
		}
		else
		{
			result = result * base + number;
		}

		character++;
		number = CheckAlpha(*character);
	}

	if (pEnd != 0)
	{
		*pEnd = (char*)character;
	}

	if (sign != 0)
	{
		result = (unsigned long)(-1 * (long)result);
	}

	return result;
}

/*********************************************************/
/*                                                       */
/* Static function implementation                        */
/*                                                       */
/*********************************************************/
/**
 * @brief  Check character is digit or not(isdigit)
 * @param  checkc character
 * @return result
 * @retval is digit  result=1
 * @retval not digit result=0
 */
static int CheckDigit(
	const char character
)
{
	if(('0' > character) || (character > '9'))
	{
		return 0;
	}
	else
	{
		return 1;
	}

#if 0
	switch(character)
	{
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
		return 1;
		break;
	default:
		return 0;
		break;
	}
#endif
}

/**
 * @brief  Check character is digit or not(isdigit)
 * @param  checkc character
 * @return result
 * @retval is digit or alpha  result=num
 * @retval not digit or alpha result=-1
 */
static int CheckAlpha(
	const char character
)
{
	if(('0' <= character) && (character <= '9'))
	{

		return (character - '0');
	}
	else if(('a' <= character) && (character <= 'z'))
	{
		return (character - 'a' + 10);
	}
	else if(('A' <= character) && (character <= 'Z'))
	{
		return (character - 'A' + 10);
	}
	else
	{
		return -1;
	}
}

/**
 * @brief  get string 
 * @param  convert string
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int GetSignstring(
	const char *pString,
	const char **endPointer
)
{
	int sign;
	const char *character;
	unsigned char isSpace;

	character = pString;
	isSpace = 1;
	sign = 0;

	while(isSpace)
	{
		switch(*character)
		{
		case ' ':
		case '\f':
		case '\n':
		case '\r':
		case '\t':
		case '\v':
			character++;
			break;
		default:
			isSpace = 0;
			break;
		}
	}

	switch(*character)
	{
	case '-':
		sign = -1;
	case '+':
		character++;
		break;
	default:
		break;
	}

	*endPointer = character;

	return sign;
}

#endif  /* PAFEE_LIB_STDLIB_ENABLE */
