/**
 *     project     PAFEE
 *      @file:     pafee_string.c
 *      @author:   Technocrats
 *      @brief:    Library Functions
 */

#include "pafee_string.h"

#ifdef PAFEE_LIB_STRING_ENABLE

/**
 * @brief  pafee_memcmp
 * @param  pBuffer1 : pointer to buffer to compare
 * @param  pBuffer2 : pointer to another buffer to compare
 * @param  count : bytes to compare
 * @return result of comparison
 * @retval zero : pBuffer1 and pBuffer2 are same
 * @retval non zero : pBuffer1 and pBuffer2 are NOT same
 */
int pafee_memcmp(
	const void *pBuffer1,
	const void *pBuffer2,
	size_t count
)
{
	register const unsigned char *pWorkBuffer1, *pWorkBuffer2, *pEnd;
	int result = 0;
	for(pWorkBuffer1 = pBuffer1, pWorkBuffer2 = pBuffer2, pEnd = pWorkBuffer2 + count;
		(pWorkBuffer2 != pEnd) && ((result = *pWorkBuffer1 - *pWorkBuffer2) == 0);
		pWorkBuffer1++, pWorkBuffer2++)
	{
	}
	return result;
}

/**
 * @brief  pafee_memcpy
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @param  count : bytes to copy
 * @return (none)
 */
void *pafee_memcpy(
	void *pTarget,
	const void *pSource,
	size_t count
)
{
	register char *pWorkTarget = pTarget;
	register const char *pWorkSource = pSource;
	register const char *pEnd;

	if(count != 0)
	{
		pEnd = pWorkSource + count;
		do
		{
			*pWorkTarget++ = *pWorkSource++;
		}
		while(pWorkSource != pEnd);
	}
	return pTarget;
}

/**
 * @brief  pafee_memmove
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @param  count : bytes to move
 * @return (none)
 */
void *pafee_memmove(
	void *pTarget,
	const void *pSource,
	size_t count
)
{
	register char *pWorkTarget = pTarget;
	register const char *pWorkSource = pSource;
	register const char *pEnd;

	if(count != 0)
	{
		if(pWorkTarget < pWorkSource)
		{
			pEnd = pWorkSource + count;
			do
			{
				*pWorkTarget++ = *pWorkSource++;
			}
			while(pWorkSource != pEnd);
		}
		else if(pWorkTarget > pWorkSource)
		{
			pEnd = pWorkSource;
			pWorkTarget += count;
			pWorkSource += count;
			do
			{
				*--pWorkTarget = *--pWorkSource;
			}
			while(pWorkSource != pEnd);
		}
	}
	return pTarget;
}

/**
 * @brief  pafee_memset
 * @param  pTarget : pointer to destination buffer
 * @param  value : value to set to buffer
 * @param  count : bytes to set value
 * @return (none)
 */
void *pafee_memset(
	void *pTarget,
	int value,
	unsigned int count
)
{
	register unsigned int string, register32, register8, toAlign;
	register unsigned int *pInt;
	register unsigned char *pChar;

	/* avoid unexpected behavior caused by memory alignment */
	pChar = pTarget;
	toAlign = 8 - ((unsigned int)pTarget & 0x00000007);  /* gap between pTarget and 8-byte alignment */
	if(toAlign != 8)
	{
		for(; toAlign>0 && count>0; toAlign--, count--)
		{
			*pChar++ = value;
		}
	}

	register32 = count >> 2;         /* bytes / 4 */
	register8 = count & 0x00000003;  /* bytes % 4 */

	string = (value << 8) | value;
	string = (string << 16) | string;  /* 4 "value"s in row */

	for(pInt = (unsigned int *)pChar; register32>0; register32--)
	{
		*pInt++ = string;
	}
	for(pChar = (unsigned char *)pInt; register8>0; register8--)
	{
		*pChar++ = string;
	}
	
	return pTarget;
}

/**
 * @brief  pafee_strcat
 * @param  pTarget : pointer to destination buffer
 * @return pSource : pointer to source buffer
 * @retval pointer to destination buffer
 */
char *pafee_strcat(
	char *pTarget,
	const char *pSource
)
{
	register char *pWorkTarget;
	for(pWorkTarget = pTarget; *pWorkTarget != '\0'; pWorkTarget++)
	{
	}

	for(; (*pWorkTarget = *pSource) != '\0'; pWorkTarget++, pSource++)
	{
	}
	return pTarget;
}

/**
 * @brief  pafee_strncat
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @param  count : bytes to concatenate
 * @return pointer to destination buffer
 */
char *pafee_strncat(
	char *pTarget,
	const char *pSource,
	size_t count
)
{
	register char *pWorkTarget, *pEnd;
	for(pWorkTarget = pTarget; *pWorkTarget != '\0'; pWorkTarget++)
	{
	}
	for(pEnd = pWorkTarget + count;
		(pWorkTarget != pEnd) && ((*pWorkTarget = *pSource) != '\0');
		pWorkTarget++, pSource++)
	{
	}
	if (pWorkTarget == pEnd)
	{
		*pWorkTarget = '\0';
	}
	return pTarget;
}

/**
 * @brief  pafee_strcpy
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @return pointer to destination buffer
 */
char *pafee_strcpy(
	char *pTarget,
	const char *pSource
)
{
	register char *pWorkTarget;
	for(pWorkTarget = pTarget;
		(*pWorkTarget = *pSource) != '\0';
		pWorkTarget++, pSource++)
	{
	}
	return pTarget;
}

/**
 * @brief  pafee_strncpy
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @param  count : bytes to copy
 * @retval pointer to destination buffer
 */
char *pafee_strncpy(
	char *pTarget,
	const char *pSource,
	size_t count
)
{
	register char *pWorkTarget, *pEnd;
	for(pWorkTarget = pTarget, pEnd = pWorkTarget + count;
		(pWorkTarget != pEnd) && ((*pWorkTarget = *pSource) != '\0');
		pWorkTarget++, pSource++)
	{
	}
	for(; pWorkTarget != pEnd; pWorkTarget++)
	{
		*pWorkTarget = '\0';
	}
	return pTarget;
}

/**
 * @brief  pafee_strlen
 * @param  pString : buffer to string
 * @return length of the string
 */
size_t pafee_strlen(
	const char *pString
)
{
	register const char *pWorkString;
	for(pWorkString = pString; *pWorkString != '\0'; pWorkString++)
	{
	}
	return (pWorkString - pString);
}

/**
 * @brief  pafee_strcmp
 * @param  pBuffer1 : pointer to buffer to compare
 * @param  pBuffer2 : pointer to another buffer to compare
 * @return result of comparison
 * @retval zero : pBuffer1 and pBuffer2 are same
 * @retval non zero : pBuffer1 and pBuffer2 are NOT same
 */
int pafee_strcmp(
	const char *pBuffer1,
	const char *pBuffer2
)
{
	register const unsigned char *pWorkBuffer1, *pWorkBuffer2;
	for(pWorkBuffer1 = (const unsigned char*)pBuffer1, pWorkBuffer2 = (const unsigned char*)pBuffer2;
		(*pWorkBuffer1 == *pWorkBuffer2) && (*pWorkBuffer1 != '\0');
		pWorkBuffer1++, pWorkBuffer2++)
	{
	}
	return *pWorkBuffer1 - *pWorkBuffer2;
}

/**
 * @brief  pafee_strstr
 * @param  pString : pointer to string to be searched
 * @param  pSearch : pointer to string to search
 * @return result of search
 * @retval NULL : "search" is not found in "string"
 * @retval non NULL : pointer to head of first "search" in "string"
 */
char *pafee_strstr(
	const char *pString,
	const char *pSearch
)
{
	register const char *pWorkString, *pWorkSearch;
	char first = *pSearch++;
	if(first == '\0')
	{
		return (char*)pString;
	}

	do
	{
		if(*pString == first)
		{
			for(pWorkString = pString + 1, pWorkSearch = pSearch;
				*pWorkSearch != '\0' && *pWorkString == *pWorkSearch;
				pWorkString++, pWorkSearch++)
			{
			}
			if(*pWorkSearch == '\0')
			{
				return (char*)pString;
			}
		}
	}
	while(*pString++ != '\0');

	return NULL;
}

/**
 * @brief  pafee_strchr
 * @param  pString : pointer to string to be searched
 * @param  find : character to find
 * @return result of search
 * @retval NULL : "find" is not found in "string"
 * @retval non NULL : pointer to first "find" in "string"
 */
char *pafee_strchr(
	const char *pString,
	int find
)
{
	find = (char)find;
	do
	{
		if(*pString == find)
		{
			return (char*)pString;
		}
	}
	while (*pString++ != '\0');

	return NULL;
}

/**
 * @brief  pafee_strrchr
 * @param  pString : pointer to string to be searched
 * @param  find : character to find
 * @return result of search
 * @retval NULL : "find" is not found in "string"
 * @retval non NULL : pointer to last "find" in "string"
 */
char *pafee_strrchr(
	const char *pString,
	int find
)
{
	char *pResult = NULL;
	find = (char)find;

	do
	{
		if(*pString == find)
		{
			pResult =  (char*)pString;  /* overwrite result */
		}
	}
	while (*pString++ != '\0');

	return pResult;
}

#endif /* PAFEE_LIB_STRING_ENABLE */


/**
 * @brief  pafee_strrstr
 * @param  pString : pointer to string to be searched
 * @param  pSearch : pointer to string to search
 * @return result of search
 * @retval NULL : "search" is not found in "string"
 * @retval non NULL : pointer to head of last "search" in "string"
 */
char *pafee_strrstr(
	const char *pString,
	const char *pSearch
)
{
	register const char *pWorkString, *pWorkSearch;
	char first = *pSearch++;
	char *pResult = NULL;

	if(first == '\0')
	{
		return (char*)pString;
	}

	do
	{
		if(*pString == first)
		{
			for(pWorkString = pString + 1, pWorkSearch = pSearch;
				*pWorkSearch != '\0' && *pWorkString == *pWorkSearch;
				pWorkString++, pWorkSearch++)
			{
			}
			if(*pWorkSearch == '\0')
			{
				pResult = (char*)pString;
			}
		}
	}
	while(*pString++ != '\0');

	return pResult;
}
