//****************************************************
//
//	project		PAFEE
//!	@file:		CharacterBuffer.h
//!	@author: 
//!	@brief:		Ring buffer for character stream
//
//****************************************************
//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//




#ifndef _CHARACTER_BUFFER_H_INCLUDED_
#define _CHARACTER_BUFFER_H_INCLUDED_

#include "PafeeCommon.h"
#include "PafeeObject.h"

namespace Pafee
{

class CharacterBuffer : public Object
{
public:
#ifdef PAFEE_STATISTICS_ENABLE
	typedef struct
	{
		unsigned long	addCount;
		unsigned long	overflowCount;
		unsigned long	maxElementCount;
		unsigned long	currentElementCount;
	} STATISTICS;
#endif // PAFEE_STATISTICS_ENABLE

public:
	// constructor
	CharacterBuffer(int capacity=0, Allocator* pAllocator=NULL);

	// destructor
	~CharacterBuffer();
	
public:
	bool Initialize(int capacity, Allocator* pAllocator=NULL);

	int GetCount(void);

	bool IsEmpty(void);

	bool Enqueue(char data);

	bool Dequeue(char* pData);

	void ClearAll(void);

#ifdef PAFEE_STATISTICS_ENABLE
	void GetStatistics(CharacterBuffer::STATISTICS* pStatistics);
#endif

private:

	char* m_pCharacterBuffer;	///< characer buffer
	unsigned long m_endPoint;			///< pointer that marks end of characer buffer
	unsigned long m_capacity;			///< capacity of character buffer
	unsigned long m_readPoint;			///< read pointer
	unsigned long m_writePoint;			///< write pointer
	bool m_isInitialized;		///< Initialize flag
	
#ifdef PAFEE_STATISTICS_ENABLE
	STATISTICS	m_statistics;
#endif

#ifdef PAFEE_UNITTEST
public:
	static void UnitTest(void);
#endif
};

};	// Pafee

#endif // _CHARACTER_BUFFER_H_INCLUDED_
