//****************************************************
//
//    project   Pafee
//!   @file:    PafeeList.h
//!   @author:  M.Tanaka
//!   @brief:   List Hedder File
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _PAFEE_LIST_H_INCLUDE_
#define _PAFEE_LIST_H_INCLUDE_

#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "pafee_string.h"
#include <limits.h>

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif //PAFEE_UNITTEST

namespace Pafee
{ 

//! List
class List : public Object
{

#ifdef PAFEE_STATISTICS_ENABLE
public:
	typedef struct
	{
		int addCount;
		int deleteCount;
		int maxElementCount;
		int minElementSize;
		int maxElementSize;
		int presentElementCount;
	} STATISTICS;
	void GetStatistics(List::STATISTICS* pStatistics);
private:
	STATISTICS m_Statistics;
#endif // PAFEE_STATISTICS_ENABLE

protected:
	typedef struct tag_NODE_BOX
	{
		tag_NODE_BOX* m_pNext;
		tag_NODE_BOX* m_pPrev;
		void* m_pData;
		int m_bufferSize;
		int m_dateLength;
		bool m_continues;
	} NODE_BOX;

public:
	typedef NODE_BOX* POSITION;

public:
	List(Allocator* pAllocator=NULL);
	List(const List& lstc, Allocator* pAllocator=NULL);
	virtual ~List();
	bool Initialize(Allocator* pAllocator=NULL);
	List& operator = (const List& lstc);
	int GetCount(void)
	{
		return m_elementNum;
	}
	bool IsEmpty(void);
	void* GetWriteBuffer(int length);
	bool AddFront(const void* pData, int length);
	bool AddFront(int length);	//No copy version
	bool AddBack(const void* pData, int length);
	bool AddBack(int length); //No copy version
	int GetFrontLength(void);
	int GetBackLength(void);
	int GetLengthAt(List::POSITION pos);
	bool GetFront(void* pBuffer, int* pLength, int bufSize=INT_MAX);
	void* GetFront(int* pLength); //No copy version
	bool GetBack(void* pBuffer, int* pLength, int bufSize=INT_MAX);
	void* GetBack(int* pLength); //No copy version
	List::POSITION GetFrontPosition(void)
	{
		return m_pStart;
	}
	List::POSITION GetBackPosition(void)
	{
		return m_pEnd;
	}
	int GetNextLength(List::POSITION& pos);
	int GetPreviousLength (List::POSITION& pos);
	bool GetNext(List::POSITION& pos, void* pBuffer, int* pLength, int bufSize=INT_MAX);
	void* GetNext(List::POSITION& pos,  int* pLength); //No copy version
	bool GetPrevious(List::POSITION &pos, void* pBuffer, int* pLength, int bufSize=INT_MAX);
	void* GetPrevious(List::POSITION &pos, int* pLength); //No copy version
	bool GetAt(List::POSITION pos, void* pBuffer, int* pLength, int bufSize=INT_MAX);
	void* GetAt(List::POSITION pos, int* pLength); //No copy version
	bool SetAt(List::POSITION& pos,  const void* pData, int length);
	void RemoveFront(void);
	void RemoveBack(void);
	void RemoveAt(List::POSITION pos);
	void RemoveAll(void);

protected:
	List::POSITION CreateBox(const void* pData=NULL, int length=NULL);
	void DestroyBox(List::POSITION pos);

protected:
	List::POSITION m_pStart;
	List::POSITION m_pEnd;
	List::POSITION m_pWriteBox;
	int m_elementNum;

#ifdef PAFEE_UNITTEST
public: // unit test function
	static void UnitTest(void);
#endif // PAFEE_UNITTEST

};

};

#endif
