//****************************************************
//
//    project   Pafee
//!   @file:    PafeeStackT.h
//!   @author:  M.Mori
//!   @brief:   StackT
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef PAFEE_INCLUDE__PAFEESTACKT_H_
#define PAFEE_INCLUDE__PAFEESTACKT_H_

#include <limits.h>
#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "PafeeListT.h"
#include "pafee_string.h"

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif //PAFEE_UNITTEST

namespace Pafee
{

//! StackT
template<class T>
class StackT : public Object
{

#ifdef PAFEE_STATISTICS_ENABLE
public:
	typedef struct
	{
		int addCount;
		int deleteCount;
		int maxElementCount;
		int presentElementCount;
	} STATISTICS;
	void GetStatistics(STATISTICS* pStatistics)
	{
		m_pList->GetStatistics((struct ListT<T>::STATISTICS* )pStatistics);
	}
private:
	STATISTICS m_Statistics;
#endif // PAFEE_STATISTICS_ENABLE

public:
	StackT(
		Allocator* pAllocator=NULL ///< [in] memory allocator
	)
	{
		SetAllocator(pAllocator);
		m_pList = (ListT<T>*)Allocate(sizeof( ListT<T>));
		m_pList = new (m_pList)ListT<T>;
	}

	StackT(
		const StackT& stkc, ///< [in] initial value of Stack
		Allocator* pAllocator=NULL ///< [in] Memory allocator
	)
	{
		SetAllocator(pAllocator);
		m_pList = (ListT<T>*)Allocate(sizeof(ListT<T>));
		new (m_pList)ListT<T>(pAllocator);
		*m_pList = *stkc.m_pList;
	}

	virtual ~StackT()
	{
		m_pList->~ListT<T>();
		Deallocate(m_pList);
	}

	StackT& operator =(
		const StackT& stkc ///< [in] initial value of Queue
    )
	{
		*m_pList = *stkc.m_pList;
		return *this;
	}

	bool Pop(
		T& Buffer ///< [out] Pop data
	)
	{
		if ( m_pList->IsEmpty() )
		{
			return false;
		}
		Buffer = m_pList->GetFront();
		m_pList->RemoveFront();
		return true;
	}

	int GetCount(void)
	{
		return m_pList->GetCount();
	}

	bool IsEmpty()
	{
		return m_pList->IsEmpty();
	}

	bool Push(
		const T& Data ///< [in] Push data
	)
	{
		m_pList->AddFront( Data );
		return true;
	}

	bool Peek(
		T& Buffer ///< [in] Get data
	)
	{
		if ( m_pList->IsEmpty() )
		{
			return false;
		}
		Buffer = m_pList->GetFront();
		return true;
	}

	void RemoveFront(void)
	{
		m_pList->RemoveFront();
	}

	void RemoveAll(void)
	{
		m_pList->RemoveAll();
	}
private:
	ListT<T>* m_pList;

};

};
#endif

