//********************************************************************
//
//! 	 @file: 	PafeeLockedAllocator.h
//! 	 @author:	T.Miyawaki
//! 	 @brief:	Locked memory allocator
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _LOCKED_ALLOCATOR_H_INCLUDED_
#define _LOCKED_ALLOCATOR_H_INCLUDED_

#include "PafeeCommon.h"
#include "PafeeAllocator.h"
#include "PafeeMutex.h"
#include <cstddef>

namespace Pafee {

//! Locked Allocator
class LockedAllocator : public Allocator
{

public:
	//! Constractor
	LockedAllocator(Allocator* pAllocator=NULL);

	//! Destoractor
	virtual ~LockedAllocator();
	
	//! Initialize
	void Initialize(Allocator* pAllocator);
	
	//! Allocate memory
	virtual void* Allocate(unsigned long size);

	//! Deallocate memory
	virtual void Deallocate(void* ptr);

	//! Check given address is in own address range
	virtual bool IsOwnAddress(void* ptr);

#ifdef PAFEE_STATISTICS_ENABLE
	//! Get statistics
	virtual void GetStatistics(StreamOutput& stream);
#endif

	//! SetAllocator
	void SetAllocator(Allocator* pAllocator)
	{
		m_pAllocator = pAllocator;
	};
	
protected:
	Allocator*	m_pAllocator;
	PafeeMutex m_mutex;
	
private:
	//! Copy constractor
	LockedAllocator(const LockedAllocator& src);

	//! Operator equal
	LockedAllocator& operator=(const LockedAllocator& src);
};

};

#endif // _LOCKED_ALLOCATOR_H_INCLUDED_
