//****************************************************
//
//      project     PAFEE
//!      @file:     PafeeTokenizer.h
//!      @author:   M.Mori
//!      @brief:    Tokenizer class
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "pafee_string.h"
#include "pafee_stdlib.h"

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif //PAFEE_UNITTEST

#ifndef PAFEE_TOKENIZER
#define PAFEE_TOKENIZER
namespace Pafee
{
	const int PAFEE_STATIC_MAX_BYTE=32;

//! Tokenizer
	class Tokenizer : Object
	{
	public:
		Tokenizer(Allocator* pAllocator=NULL);
		Tokenizer(const char* szTarget, char token=',', char quote='\"', char* szEndMark="\r\n", Allocator* pAllocator=NULL);
		~ Tokenizer ();
		int Split(const char* szTarget, char token=',', char quote='\"', char* szEndMark="\r\n");
		int GetCount(void){return m_indexSize;}
		bool IsEmpty(void){return !m_indexSize;}
		bool Get(int index, char* szBuffer, int bufferSize);
		char* Get(int index);
		int GetInt(
			int index,  ///< [in] index
			int base  ///< [in] A cardinal number
		)
		{
			return pafee_strtol(m_ppData[index], NULL, base );
		}
		int GetLength(int index);

	private:
		char*  TokenFind(const char* str, char* findStr, char quote);
		char*  TokenFind(const char* str, char  findChr, char quote);
		char   m_szData[PAFEE_STATIC_MAX_BYTE];
		char*  m_pData;
		char** m_ppData;
		int    m_length;
		int    m_allocateSize;
		int    m_indexSize;
		char*  m_pStart;
	private:
		inline int AligmentSize(int size) {return ((((size)-1)|(((size)-1)^3))+1);};

#ifdef PAFEE_UNITTEST
	public:
		static void UnitTest(void);
#endif // PAFEE_UNITTEST
	};
}

#endif

