//****************************************************
//
//      project     PAFEE
//!      @file:     ClassDependency.h
//!      @author:   Technocrats
//!      @brief:    Dependence definition file analysis class header
//
//*****************************************************

#ifndef _CLASS_DEPENDENCY_H_INCLUDED
#define _CLASS_DEPENDENCY_H_INCLUDED

#include <string>
#include <list>
#include <queue>
#include "define.h"

using namespace std;

#ifdef __cplusplus
namespace Pafee {
#endif
class ClassDependency{
private:
	list<string> m_classTable; /**< Use category/class name */
	list<string> m_dependencyTable; /**< Depend file list */
	list<string> m_useClassList; /**< Already exist class name list*/
	queue<string> m_findClassList; /**< Next find class */

	string m_filePath; /**< Dependency file path */

	/**
	 * @brief  check class use file and other class
	 * @param  classNum number of PAFEE_CLASS_TABLE
	 * @return Read success or not
	 * @retval TRUE Read success
	 * @retval FALSE Read failure
	 */
	BOOL ClassFileChecker(
		const string categoryClassName
	);

	/**
	 * @brief  check class name folder
	 * @param  class name folder path
	 * @return Read success or not
	 * @retval TRUE Read success
	 * @retval FALSE Read failure
	 */
	BOOL ClassFolderChecker(
		const string classFolderName
	);

	/**
	 * @brief  check .h is used files
	 * @param  Header file name string
	 * @return Read success or not
	 * @retval TRUE Read success
	 * @retval FALSE Read failure
	 */
	BOOL HeaderFileChecker(
		const string headerFileName
	);

	/**
	 * @brief copy constractor of ClassDependency Class
	 */
	ClassDependency(const ClassDependency& org);

	/**
	 * @brief operator: equal
	 */
	ClassDependency& operator=(const ClassDependency& org);
public:

	/**
	 * @brief constractor of ClassDependency Class
	 * @param filePath Depend file path
	 */
	ClassDependency(const string filePath="");

	/**
	 * @brief destoractor of ClassDependency Class
	 */
	~ClassDependency(); 

	/**
	 * @brief  Depend file read method
	 * @param  filePath Depend file path
	 * @return Read success or not
	 * @retval TRUE Read success
	 * @retval FALSE Read failure
	 */
	BOOL ReadFile(
		const string filePath
	);

	/**
	 * @brief  Analyze dependency
	 * @param  pClassTable Using class define table
	 * @return Analysis success or not
	 * @retval TRUE Analysis success
	 * @retval FALSE Analysis failure
	 */
	BOOL DefineDependency(
		const list<string> classTable,
		const BOOL isFreeStanding=FALSE
	);

	/**
	 * @brief  Get dependency
	 * @param  pDependencyTable Pointer of a buffer storing away dependence
	 * @return null
	 */
	BOOL GetDependency(
		list<string> *pDependencyTable
	);
};

#ifdef __cplusplus
}; // namespace Pafee
#endif
#endif // _CLASS_DEPENDENCY_H_INCLUDED
