//****************************************************
//
//      project     PAFEE
//!      @file:     FileCollector.h
//!      @author:   Technocrats
//!      @brief:    File collector class header
//
//*****************************************************
#include "StdAfx.h"
#include "define.h"
#include "assert.h"
#include "FileCollector.h"
#include <iostream>
#include <windows.h>

using namespace std;

namespace Pafee{
static const string g_defaultPath = DEFAULT_OUT_PATH;
static const string g_defaultSourcePath = DEFAULT_SOURCE_PATH;
static const string g_defaultPathDelimitor = "/\\";
/**
 * @brief copy constractor of FileCollector Class
 */
FileCollector::FileCollector(
	const FileCollector& org
)
{
}

/**
 * @brief operator: equal
 */
FileCollector& FileCollector::operator=(
	const FileCollector& org
)
{
	return *this;
}

/**
 * @brief constractor of FileCollector Class
 */
FileCollector::FileCollector(
)
{
	SetSourcePath(g_defaultSourcePath);
	SetOutputPath(g_defaultPath);

	m_isOverwrite = TRUE;
}

/**
 * @brief destoractor of FileCollector Class
 */
FileCollector::~FileCollector(
)
{
}


/**
 * @brief  Select PAFEE source folder
 * @param  sourcePath PAFEE source path
 * @return non
 */
void FileCollector::SetSourcePath(
	const string sourcePath
)
{
	string::size_type size;

	size = sourcePath.size();

	if(size == 0)
	{
		m_sourcePath = "./";
	}
	else if((sourcePath[size-1] == '/') || (sourcePath[size-1] == '\\'))
	{
		m_sourcePath = sourcePath;
	}
	else
	{
		m_sourcePath = sourcePath + '/';
	}
}

/**
 * @brief  Select PAFEE output folder
 * @param  sourcePath PAFEE source path
 * @return non
 */
void FileCollector::SetOutputPath(
	const string outputPath
)
{
	string::size_type size;

	size = outputPath.size();

	if(size == 0)
	{
		m_outputPath = "./";
	}
	else if((outputPath[size-1] == '/') || (outputPath[size-1] == '\\'))
	{
		m_outputPath = outputPath;
	}
	else
	{
		m_outputPath = outputPath + '/';
	}
}

/**
 * @brief  Output files in appointed folder
 * @param  outputPath Output folder path
 * @return Output success or not
 * @retval true Output success
 * @retval false Output failure
 */
BOOL FileCollector::PutFile(
	const list<string> fileTable
)
{
	string sourcePath;
	string outputFolder;
	list<string>::iterator stringIterator;
	FILE *fp=NULL;
	string::size_type size;
	BOOL retval=TRUE;
	BOOL isSource=TRUE;
	BOOL isFolder=TRUE;
	BOOL isFile=FALSE;
	BOOL canCopy=TRUE;

	m_fileTable = fileTable;
	while(!m_makeList.empty())
	{
		m_makeList.pop();
	}

	// Check source file
	stringIterator = m_fileTable.begin();
	while(stringIterator != m_fileTable.end())
	{
		sourcePath = m_sourcePath + *stringIterator;
		stringIterator++;

		if ((fp = fopen(sourcePath.c_str(), "r")) == NULL)
		{
			isSource = FALSE;
			break;
		}
		fclose(fp);
	}

	if(isSource == TRUE)
	{
		// Cleate category folder
		stringIterator = m_fileTable.begin();
		while(stringIterator != m_fileTable.end())
		{
			sourcePath = m_outputPath + *stringIterator;
			stringIterator++;
			size = sourcePath.find_last_of(g_defaultPathDelimitor);
			outputFolder = sourcePath.substr(0, size+1);
//			outputFolder = sourcePath.substr(0, size+1);
//			CreateDirectory(outputFolder.c_str(), NULL);
			CreateFolder(outputFolder.c_str());
		}

		// Check Output file
		stringIterator = m_fileTable.begin();
		while(stringIterator != m_fileTable.end())
		{
			sourcePath = m_sourcePath + *stringIterator;
			outputFolder = m_outputPath + *stringIterator;
			stringIterator++;

			if(m_isOverwrite == TRUE)
			{
				canCopy = CopyFile(sourcePath.c_str(), outputFolder.c_str(), FALSE);
			}
			else
			{
				if ((fp = fopen(outputFolder.c_str(), "r")) == NULL)
				{
					isFile = FALSE;
					break;
				}
				else
				{
					isFile = TRUE;
				}

				fclose(fp);

				if(isFile == TRUE)
				{
					canCopy = TRUE;
				}
				else
				{
					canCopy = CopyFile(sourcePath.c_str(), outputFolder.c_str(), TRUE);
				}
			}

			if(canCopy == FALSE)
			{
				cout << "Can not copy file : " << outputFolder << "\n";

				// delete maked file
				while(!m_makeList.empty())
				{
					outputFolder = m_makeList.top();
					m_makeList.pop();

					if(outputFolder.find_last_of(g_defaultPathDelimitor) == (outputFolder.length() - 1))
					{
						RemoveDirectory(outputFolder.c_str());
					}
					else
					{
						remove(outputFolder.c_str());
					}
				}
				break;
			}
			else
			{
				m_makeList.push(outputFolder);
			}
		}
	}
	else
	{
		cout << "Can not find file : " << sourcePath << "\n";
	}

	return retval;
}

/**
 * @brief  Create folder. If there is, not create
 * @param  folderPath Creating folder path
 * @return Create success or not
 * @retval true Create success
 * @retval false Createt failure
 */
BOOL FileCollector::CreateFolder(
	const string folderPath
)
{
	string tmpFolderPath;
	string makeDirectory;
	string::size_type size;
	BOOL retval= TRUE;
	BOOL canMake;

	tmpFolderPath = folderPath;
	while(tmpFolderPath != "")
	{
		size = tmpFolderPath.find_first_of(g_defaultPathDelimitor);
		if(size == string::npos)
		{
			break;
		}

		makeDirectory = makeDirectory+tmpFolderPath.substr(0, size+1);
		tmpFolderPath = tmpFolderPath.substr(size+1);
		canMake = CreateDirectory(makeDirectory.c_str(), NULL);
		if(canMake == TRUE)
		{
			m_makeList.push(folderPath);
		}
	}
	return retval;
}
}; // namespace Pafee
