//****************************************************
//
//      project    PAFEE
//!      @file:     PafeeBuildToolMain.cpp
//!      @author:   Technocrats
//!      @brief:    Main process of PAFEE Build Environment Structure Tool
//
//*****************************************************

#include "StdAfx.h"
#include "PafeeBuildToolMain.h"

using namespace std;
using namespace Pafee;

/**
 * @brief Print help massage
 */
static void PrintHelp(
 void
)
{
	printf("pafeeConfig.exe [-options]\n");
	printf("\n");
	printf("where options include:\n");
	printf("-i <config file path>\t\t set config file path\n");
	printf("-d <dependency file path>\t set dependency file path\n");
	printf("-o <output folder path>\t\t set output folder path\n");
	printf("-s <source tree path>\t\t PAFEE source tree folder path\n");
	printf("-freestanding\t\t\t enable C library subset\n");
	printf("-v\t\t\t\t print version information\n");
	printf("-h\t\t\t\t print help message\n");
}

/**
 * @brief Main process of PAFEE Build Environment Structure Tool
 */
int PafeeBuildToolMain(
	int argc,
	char* argv[]
)
{
	int result = 0;
	CommandOption commandOption(argc, (const char **)argv);

	if(commandOption.IsWrongInput() == TRUE)
	{
		printf("Incorrect command line input\n");
		printf("\n");
		PrintHelp();
		return 0;
	}

	if(commandOption.IsVersion() == TRUE)
	{
		printf("PAFEE Ver.1.0\n");
		return 0;
	}
	else if(commandOption.IsHelp() == TRUE)
	{
		PrintHelp();
		return 0;
	}

	printf("collecting source files...\n");

	list<string> usingClassList;
	list<string> usingFileList;

	StructureDefinition structureDefinition(commandOption.GetConfigFilePath());
	result = structureDefinition.DefineStructure();
	if(result == FALSE)
	{
		return 0;
	}
	result = structureDefinition.GetClassList(&usingClassList);
	if(result == FALSE)
	{
		return 0;
	}
	ClassDependency classDependency(commandOption.GetDependencyFilePath());
	result = classDependency.DefineDependency(usingClassList, commandOption.IsFreestanding());
	if(result == FALSE)
	{
		return 0;
	}
	result = classDependency.GetDependency(&usingFileList);
	if(result == FALSE)
	{
		return 0;
	}
	FileCollector fileCollector;
	fileCollector.SetSourcePath(commandOption.GetSourceTreePath());
	fileCollector.SetOutputPath(commandOption.GetOutputPath());
	result = fileCollector.PutFile(usingFileList);
	if(result == FALSE)
	{
		return 0;
	}

	printf("done,\n");		// completed

	return 0;
}


/**
 * @brief Main function of PAFEE Build Environment Structure Tool
 */
#if 0
int main(
	int argc,
	char* argv[]
)
{
	PafeeBuildToolMain(argc, argv);
	return 0;
}
#endif
