//****************************************************
//
//      project     PAFEE
//!      @file:     PafeeCrc.h
//!      @author:   chu
//!      @brief:    Crc class header file
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _Crc_H_INCLUDED_
#define _Crc_H_INCLUDED_

#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "pafee_string.h"

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif

namespace Pafee
{

//------------------------------------------------------------------------------
//!    - @brief    This class supports CRC calculation function
//!        - support for CRC16(CCITT) and CRC32 calculation function
//!    .
//------------------------------------------------------------------------------
//! Crc
class Crc : public Object
{
public:
	Crc();
	~Crc();
	static unsigned short Encode16(void* pData, int length);
	static unsigned long Encode32(void* pData, int length);
private:
	static unsigned long Reflect(unsigned long data, unsigned char reflectSize);	
	static unsigned short Reflect(unsigned short data, unsigned char reflectSize);	
	static const unsigned long CRC_POLYNOMIAL;
	static const unsigned short CRC16_POLYNOMIAL;

#ifdef PAFEE_UNITTEST
public: // unit test function
	static void UnitTest(void);
	// Test for Encode16
	bool IsCrc16Good(void* pData, int length);
#endif // PAFEE_UNITTEST
	
};

}	// Pafee

#endif
