//****************************************************
//
//  project   PAFEE
//! @file:    PafeeLog.cpp
//! @author:  Pengkai Cheng
//! @brief:   Log management class
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifdef ENV_WIN32
#pragma warning(disable:4996)
#endif

#include "PafeeLog.h"

using namespace std;

namespace Pafee
{

//--------------------------------------
//! constructor
//--------------------------------------
Log::Log(
	StreamOutput* pOutput ///<[in] File stream
)
{
	if(pOutput == NULL)
	{
		m_pOutput = NULL;
	}
	else
	{
		m_pOutput = pOutput;
	}
	for(int i = 0; i < 8; i++)
	{
		m_pLevelString[i] = new char[3];
	}
	for(int j = 0; j < 32; j++)
	{
		m_pCategoryString[j] = new char[3];
	}
}

//--------------------------------------
//! destructor
//--------------------------------------
Log::~Log()
{
//	m_pOutput->close();
	for(int i = 0; i < 8; i++)
	{
		delete m_pLevelString[i];
	}
	for(int j = 0; j < 32; j++)
	{
		delete m_pCategoryString[j];
	}
}

//--------------------------------------
//!	log input
//--------------------------------------
void Log::Write(
	int level,            ///<[in] Log level
	int category,         ///<[in] Log category 
	const char* stringLog ///<[in] Write string
)
{
	if(level <= m_level)
	{
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//! log input(version of string)
//--------------------------------------
void Log::Write(
	int level,              ///<[in] Log level
	int category,           ///<[in] Log category
	const char* stringLog,  ///<[in] Write string
	char* p1, ///<[in] String argument
	char* p2, ///<[in] String argument
	char* p3, ///<[in] String argument
	char* p4  ///<[in] String argument
)
{
	if(level <= m_level)
	{
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("(", 1);
		int length = pafee_strlen(p1);
		m_pOutput->Write(p1,length);
		if(p2 != NULL)
		{
			m_pOutput->Write(",", 1);
			length = pafee_strlen(p2);
			m_pOutput->Write(p2,length);
		}
		if(p3 != NULL)
		{
			m_pOutput->Write(",", 1);
			length = pafee_strlen(p3);
			m_pOutput->Write(p3,length);
		}
		if(p4 != NULL)
		{
			m_pOutput->Write(",", 1);
			length = pafee_strlen(p4);
			m_pOutput->Write(p4,length);
		}
		m_pOutput->Write(")", 1);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//!	log input(version of integer)
//--------------------------------------
void Log::Write(
	int level,             ///<[in] Log level
	int category,          ///<[in] Log category
	const char* stringLog, ///<[in] Write string
	int n1 ///<[in] Numerical value argument
)
{
	if(level <= m_level)
	{
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("(", 1);
		IntegerWrite(n1);
		m_pOutput->Write(")", 1);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//! log input(version of integer)
//--------------------------------------
void Log::Write(
	int level,             ///<[in] Log level
	int category,          ///<[in] Log category
	const char* stringLog, ///<[in] Write string
	int n1, ///<[in] Numerical value argument
	int n2  ///<[in] Numerical value argument
)
{
	if(level <= m_level)
	{
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("(", 1);
		IntegerWrite(n1);
		m_pOutput->Write(",", 1);
		IntegerWrite(n2);
		m_pOutput->Write(")", 1);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//! log input(version of integer)
//--------------------------------------
void Log::Write(
	int level,             ///<[in] Log level
	int category,          ///<[in] Log category
	const char* stringLog, ///<[in] Write string
	int n1, ///<[in] Numerical value argument
	int n2, ///<[in] Numerical value argument
	int n3  ///<[in] Numerical value argument
)
{
	if(level <= m_level)
	{
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("(", 1);
		IntegerWrite(n1);
		m_pOutput->Write(",", 1);
		IntegerWrite(n2);
		m_pOutput->Write(",", 1);
		IntegerWrite(n3);
		m_pOutput->Write(")", 1);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//! log input(version of integer)
//--------------------------------------
void Log::Write(
	int level,             ///<[in] Log level
	int category,          ///<[in] Log category
	const char* stringLog, ///<[in] Write string
	int n1, ///<[in] Numerical value argument
	int n2, ///<[in] Numerical value argument
	int n3, ///<[in] Numerical value argument
	int n4  ///<[in] Numerical value argument
)
{
	if(level <= m_level)
	{
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("(", 1);
		IntegerWrite(n1);
		m_pOutput->Write(",", 1);
		IntegerWrite(n2);
		m_pOutput->Write(",", 1);
		IntegerWrite(n3);
		m_pOutput->Write(",", 1);
		IntegerWrite(n4);
		m_pOutput->Write(")", 1);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//! log input(version of hex)
//--------------------------------------
void Log::WriteX(
	int level,             ///<[in] Log level
	int category,          ///<[in] Log category
	const char* stringLog, ///<[in] Write string
	int n1, ///<[in] Numerical value argument(Hexadecimal expression)
	int n2  ///<[in] Numerical value argument(Hexadecimal expression)
)
{
	if(level <= m_level)
	{
		
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("(", 1);
		HexWrite(n1);
		m_pOutput->Write(",", 1);
		HexWrite(n2);
		m_pOutput->Write(")", 1);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//! log input(version of hex)
//--------------------------------------
void Log::WriteX(
	int level,             ///<[in] Log level
	int category,          ///<[in] Log category
	const char* stringLog, ///<[in] Write string
	int n1, ///<[in] Numerical value argument(Hexadecimal expression)
	int n2, ///<[in] Numerical value argument(Hexadecimal expression)
	int n3  ///<[in] Numerical value argument(Hexadecimal expression)
)
{
	if(level <= m_level)
	{
		
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("(", 1);
		HexWrite(n1);
		m_pOutput->Write(",", 1);
		HexWrite(n2);
		m_pOutput->Write(",", 1);
		HexWrite(n3);
		m_pOutput->Write(")", 1);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//! log input(version of hex)
//--------------------------------------
void Log::WriteX(
	int level,             ///<[in] Log level
	int category,          ///<[in] Log category
	const char* stringLog, ///<[in] Write string
	int n1, ///<[in] Numerical value argument(Hexadecimal expression)
	int n2, ///<[in] Numerical value argument(Hexadecimal expression)
	int n3, ///<[in] Numerical value argument(Hexadecimal expression)
	int n4  ///<[in] Numerical value argument(Hexadecimal expression)
)
{
	if(level <= m_level)
	{
		
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("(", 1);
		HexWrite(n1);
		m_pOutput->Write(",", 1);
		HexWrite(n2);
		m_pOutput->Write(",", 1);
		HexWrite(n3);
		m_pOutput->Write(",", 1);
		HexWrite(n4);
		m_pOutput->Write(")", 1);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//! binary dump output
//--------------------------------------
void Log::Dump(
	int level,             ///<[in] Log level
	int category,          ///<[in] Log category
	const char* stringLog, ///<[in] Write string
	void* pData,           ///<[in] Log Data
	int length             ///<[in] Data length
)
{
	m_pData = (unsigned char *)pData;
	char buf[10];
	if(level <= m_level)
	{
		BasicWrite(level, category, stringLog);
		m_pOutput->Write("(", 1);
		for(int i = 0; i < length; i++)
		{
			//itoa(m_pData[i], buf, 16);
			pafee_sprintf(buf,"%X",m_pData[i]);
			int j;
			for( j = 0; buf[j] != '\0'; j++);
			m_pOutput->Write(buf, j);
		}
		m_pOutput->Write(")", 1);
		m_pOutput->Write("\n", 1);
	}
}

//--------------------------------------
//! set the log level string
//--------------------------------------
void Log::SetLevelString(
	int level,       ///<[in] Log level
	char* levelString ///<[in] Log level string
)
{
	if(level < 8)
	{
		int n = pafee_strlen(levelString);
		if(n > 16)
		{
			n = 16;
		}
		delete[] m_pLevelString[level];
		m_pLevelString[level] = new char[n + 1];
		pafee_strncpy(m_pLevelString[level], levelString, n);
		m_pLevelString[level][n] = '\0';
	}
}

//--------------------------------------
//! set the category string
//--------------------------------------
void Log::SetCategoryString(
	int category,        ///<[in] Log category
	char* categoryString ///<[in] Category  string
)
{
	if(category < 32)
	{
		int n = pafee_strlen(categoryString);
		if(n > 16)
		{
			n = 16;
		}
		delete[] m_pCategoryString[category];
		m_pCategoryString[category] = new char[n + 1];
		pafee_strncpy(m_pCategoryString[category], categoryString, n);
		m_pCategoryString[category][n] = '\0';
	}
}

//---------------------------------------------------------------------
//! set log level(only output the log witch belows the specified level)
//---------------------------------------------------------------------
void Log::SetLogLevel(
	int level ///<[in] Log level
)
{
	m_level = level;
	if(level >= 8)
	{
		m_level = 7;
	}
}

//--------------------------------------
//! set the path of log output
//--------------------------------------
void Log::SetLogOutput(
	StreamOutput* pOutput ///<[in] File stream
)
{
	m_pOutput = pOutput;
}

//--------------------------------------
//! BasicWrite
//--------------------------------------
void Log::BasicWrite(
	int level,             ///<[in] Log level
	int category,          ///<[in] Log category
	const char* stringLog  ///<[in] Write string
)
{
	if(*(int *)m_pLevelString[level] > 0)
	{
		int n = pafee_strlen(m_pLevelString[level]);
		m_pOutput->Write("[", 1);
		m_pOutput->Write(m_pLevelString[level], n);
		m_pOutput->Write("]", 1);
	}
	else
	{
		m_pOutput->Write("[ ]", 3);
	}
	if(*(int *)m_pCategoryString[category] > 0)
	{
		int n = pafee_strlen(m_pCategoryString[category]);
		m_pOutput->Write("[", 1);
		m_pOutput->Write(m_pCategoryString[category], n);
		m_pOutput->Write("]", 1);
	}
	else
	{
		m_pOutput->Write("[ ]", 3);
	}
	int length = pafee_strlen(stringLog);
	m_pOutput->Write(stringLog, length);
}

//--------------------------------------
//! IntegerWrite
//--------------------------------------
void Log::IntegerWrite(
	int IntWriteData  ///<[in] Write integer
)
{
	char buf[10];
	pafee_sprintf(buf,"%d", IntWriteData);
	m_pOutput->Write(buf);
/*
	int i = 0;
	do
	{
		buf[i] = IntWriteData%10 + 48;
		i++;
		IntWriteData = IntWriteData/10;
	}
	while(IntWriteData != 0);
	for(; i > 0; i--)
	{
		m_pOutput->Write(&buf[i - 1], 1);
	}
*/
}

//--------------------------------------
//! HexWrite
//--------------------------------------
void Log::HexWrite(
	int HexWriteData ///<[in] Write hex
)
{
	char buf[10];
	//itoa(HexWriteData, buf, 16);
	pafee_sprintf(buf, "%X",HexWriteData);
	int i;
	for(i = 0; buf[i] != '\0'; i++);
	m_pOutput->Write(buf, i);
}

#ifdef PAFEE_UNITTEST
//----------------------------------------------------------------------------
//! Unit test
//----------------------------------------------------------------------------
void Log::UnitTest()
{
	int i;
	char a;
	char str[] = "dump1";
	//
	//Log testLog(pOutput);
	Log testLog;
	StdStreamOutput pOutput;
	testLog.SetLogOutput(&pOutput);
	//test pattern 1
	testLog.SetLevelString(0, "LevelA");
	testLog.SetCategoryString(0, "Cat0");
	testLog.SetLogLevel(0);
	pOutput.Write("SetLevelString(0, \"LevelA\")\n") ;
	pOutput.Write("SetCategoryString(0, \"Cat0\")\n");
	pOutput.Write( "SetLogLevel(0)\n");
	pOutput.Write( "Write(0, 0, \"Level0_Categ0\")\n");
	testLog.Write(0, 0, "Level0_Categ0");
	pOutput.Write( "Write(1, 1, \"Level1_Categ1\")\n");
	testLog.Write(1, 1, "Level1_Categ1");
	pOutput.Write( "Write(7, 31, \"Level7_Categ31\")\n");
	testLog.Write(7, 31, "Level7_Categ31");
	pOutput.Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest1A\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest1A");
	pOutput.Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest1A\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest1A");
	pOutput.Write( "Write(8, 31, \"Level7_Categ31\", \"Logtest1A\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest1A");
	pOutput.Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest2A", "Logtest2B");
	pOutput.Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest2A", "Logtest2B");
	pOutput.Write("Write(7, 31, \"Level7_Categ31\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest2A", "Logtest2B");
	pOutput.Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput.Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput.Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput.Write("Write(0, 0, \"Level0_Categ0\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput.Write("Write(1, 1, \"Level1_Categ1\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput.Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput.Write("Write(0, 0, \"Level0_Categ0\", 10)\n");
	testLog.Write(0, 0, "Level0_Categ0", 10);
	pOutput.Write("Write(1, 1, \"Level1_Categ1\", 10 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 10 );
	pOutput.Write( "Write(7, 31, \"Level7_Categ31\", 10 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 10 );
	pOutput.Write("Write(0, 0, \"Level0_Categ0\", 20, 200 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 20, 200 );
	pOutput.Write( "Write(1, 1, \"Level1_Categ1\", 20, 200 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 20, 200 );
	pOutput.Write( "Write(7, 31, \"Level7_Categ31\", 20, 200 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 20, 200 );
	pOutput.Write( "Write(0, 0, \"Level0_Categ0\", 30, 300, 3000 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 30, 300, 3000 );
	pOutput.Write( "Write(1, 1, \"Level1_Categ1\", 30, 300, 3000 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 30, 300, 3000 );
	pOutput.Write("Write(7, 31, \"Level7_Categ31\", 30, 300, 3000 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 30, 300, 3000 );
	pOutput.Write( "Write(0, 0, \"Level0_Categ0\", 40, 400, 4000, 40000 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 40, 400, 4000, 40000 );
	pOutput.Write( "Write(1, 1, \"Level1_Categ1\", 40, 400, 4000, 40000 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 40, 400, 4000, 40000 );
	pOutput.Write("Write(7, 31, \"Level7_Categ31\", 40, 400, 4000, 40000 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 40, 400, 4000, 40000 );
	pOutput.Write( "WriteX(0, 0, \"Level0_Categ0\", 0x14, 0xC8 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x14, 0xC8 );
	pOutput.Write( "WriteX(1, 1, \"Level1_Categ1\", 0x14, 0xC8 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x14, 0xC8 );
	pOutput .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x14, 0xC8 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x14, 0xC8 );
	pOutput.Write("WriteX(0, 0, \"Level0_Categ0\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x1E, 0x12C, 0xBB8 );
	pOutput.Write( "WriteX(1, 1, \"Level1_Categ1\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x1E, 0x12C, 0xBB8 );
	pOutput.Write("WriteX(7, 31, \"Level7_Categ31\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x1E, 0x12C, 0xBB8 );
	pOutput.Write("WriteX(0, 0, \"Level0_Categ0\", 0x28, 0x190, 0xFA0, 0x9C40 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x28, 0x190, 0xFA0, 0x9C40 );
	pOutput.Write( "WriteX(1, 1, \"Level1_Categ1\", 0x28, 0x190, 0xFA0, 0x9C40 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x28, 0x190, 0xFA0, 0x9C40 );
	pOutput.Write("WriteX(7, 31, \"Level7_Categ31\", 0x28, 0x190, 0xFA0, 0x9C40 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x28, 0x190, 0xFA0, 0x9C40 );
	i = 0xEFCDAB12;
	pOutput.Write( "i = 0xEFCDAB12");
	pOutput.Write( "Dump(0, 0, \"Level0_Categ0_intDump\", &i, 4)\n");
	testLog.Dump(0, 0, "Level0_Categ0_intDump", &i, 4);
	pOutput.Write( "Dump(1, 1, \"Level1_Categ1_intDump\", &i, 4)\n");
	testLog.Dump(1, 1, "Level1_Categ1_intDump", &i, 4);
	pOutput.Write( "Dump(7, 31, \"Level7_Categ31_intDump\", &i, 4)\n");
	testLog.Dump(7, 31, "Level7_Categ31_intDump", &i, 4);
	a = 'a';
	pOutput.Write("a = 'a'");
	pOutput.Write( "Dump(0, 0, \"Level0_Categ0_charDump\", &a, 1)\n");
	testLog.Dump(0, 0, "Level0_Categ0_charDump", &a, 1);
	pOutput.Write( "Dump(1, 1, \"Level1_Categ1_charDump\", &a, 1)\n");
	testLog.Dump(1, 1, "Level1_Categ1_charDump", &a, 1);
	pOutput.Write( "Dump(7, 31, \"Level7_Categ31_charDump\", &a, 1)\n");
	testLog.Dump(7, 31, "Level7_Categ31_charDump", &a, 1);
	pafee_strcpy(str,"dump1");
	pOutput.Write( "pafee_strcpy(str,\"dump1\")\n");
	pOutput.Write( "Dump(0, 0, \"Level0_Categ0_stringDump\", str, 5)\n");
	testLog.Dump(0, 0, "Level0_Categ0_stringDump", str, 5);
	pOutput.Write( "Dump(1, 1, \"Level1_Categ1_stringDump\", str, 5)\n");
	testLog.Dump(1, 1, "Level1_Categ1_stringDump", str, 5);
	pOutput.Write( "Dump(7, 31, \"Level7_Categ31_stringDump\", str, 5)\n");
	testLog.Dump(7, 31, "Level7_Categ31_stringDump", str, 5);
	//test pattern 2
	StdStreamOutput pOutput2;
	testLog.SetLogOutput(&pOutput2);
	testLog.SetLevelString(0, "LevelA");	
	testLog.SetLevelString(1, "LevelB");	
	testLog.SetCategoryString(0, "Cat0");	
	testLog.SetCategoryString(1, "Cat1");	
	testLog.SetLogLevel(1);	
	pOutput2.Write( "SetLevelString(0, \"LevelA\")\n");
	pOutput2.Write( "SetLevelString(1, \"LevelB\")\n");
	pOutput2.Write( "SetCategoryString(0, \"Cat0\")\n");
	pOutput2.Write( "SetCategoryString(1, \"Cat1\")\n");
	pOutput2.Write( "SetLogLevel(1)\n");
	pOutput2.Write("Write(0, 0, \"Level0_Categ0\")\n");
	testLog.Write(0, 0, "Level0_Categ0");
	pOutput2.Write( "Write(1, 1, \"Level1_Categ1\")\n");
	testLog.Write(1, 1, "Level1_Categ1");
	pOutput2.Write( "Write(1, 2, \"Level1_Categ2\")\n");
	testLog.Write(1, 2, "Level1_Categ2");
	pOutput2.Write("Write(7, 2, \"Level7_Categ31\")\n");
	testLog.Write(7, 31, "Level7_Categ31");
	pOutput2.Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest1A\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest1A");
	pOutput2.Write("Write(1, 1, \"Level1_Categ1\", \"Logtest1A\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest1A");
	pOutput2.Write("Write(1, 2, \"Level1_Categ2\", \"Logtest1A\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest1A");
	pOutput2.Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest1A\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest1A");
	pOutput2.Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest2A", "Logtest2B");
	pOutput2.Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest2A", "Logtest2B");
	pOutput2.Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest2A", "Logtest2B");
	pOutput2.Write("Write(7, 31, \"Level7_Categ31\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest2A", "Logtest2B");
	pOutput2.Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput2.Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput2.Write("Write(1, 2, \"Level1_Categ2\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput2.Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput2.Write("Write(0, 0, \"Level0_Categ0\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput2.Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput2.Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput2.Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput2.Write( "Write(0, 0, \"Level0_Categ0\", 10)\n");
	testLog.Write(0, 0, "Level0_Categ0", 10);
	pOutput2.Write("Write(1, 1, \"Level1_Categ1\", 10 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 10 );
	pOutput2.Write( "Write(1, 2, \"Level1_Categ2\", 10 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 10 );
	pOutput2.Write( "Write(7, 31, \"Level7_Categ31\", 10 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 10 );
	pOutput2.Write("Write(0, 0, \"Level0_Categ0\", 20, 200 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 20, 200 );
	pOutput2.Write( "Write(1, 1, \"Level1_Categ1\", 20, 200 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 20, 200 );
	pOutput2.Write( "Write(1, 2, \"Level1_Categ2\", 20, 200 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 20, 200 );
	pOutput2.Write( "Write(7, 31, \"Level7_Categ31\", 20, 200 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 20, 200 );
	pOutput2.Write( "Write(0, 0, \"Level0_Categ0\", 30, 300, 3000 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 30, 300, 3000 );
	pOutput2.Write("Write(1, 1, \"Level1_Categ1\", 30, 300, 3000 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 30, 300, 3000 );
	pOutput2.Write("Write(1, 2, \"Level1_Categ2\", 30, 300, 3000 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 30, 300, 3000 );
	pOutput2.Write( "Write(7, 31, \"Level7_Categ31\", 30, 300, 3000 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 30, 300, 3000 );
	pOutput2.Write( "Write(0, 0, \"Level0_Categ0\", 40, 400, 4000, 40000 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 40, 400, 4000, 40000 );
	pOutput2.Write( "Write(1, 1, \"Level1_Categ1\", 40, 400, 4000, 40000 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 40, 400, 4000, 40000 );
	pOutput2.Write( "Write(1, 2, \"Level1_Categ2\", 40, 400, 4000, 40000 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 40, 400, 4000, 40000 );
	pOutput2.Write( "Write(7, 31, \"Level7_Categ31\", 40, 400, 4000, 40000 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 40, 400, 4000, 40000 );
	pOutput2.Write( "WriteX(0, 0, \"Level0_Categ0\", 0x14, 0xC8 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x14, 0xC8 );
	pOutput2.Write( "WriteX(1, 1, \"Level1_Categ1\", 0x14, 0xC8 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x14, 0xC8 );
	pOutput2.Write( "WriteX(1, 2, \"Level1_Categ2\", 0x14, 0xC8 )\n");
	testLog.WriteX(1, 2, "Level1_Categ2", 0x14, 0xC8 );
	pOutput2 .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x14, 0xC8 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x14, 0xC8 );
	pOutput2 .Write( "WriteX(0, 0, \"Level0_Categ0\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x1E, 0x12C, 0xBB8 );
	pOutput2 .Write( "WriteX(1, 1, \"Level1_Categ1\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x1E, 0x12C, 0xBB8 );
	pOutput2 .Write( "WriteX(1, 2, \"Level1_Categ2\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(1, 2, "Level1_Categ2", 0x1E, 0x12C, 0xBB8 );
	pOutput2 .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x1E, 0x12C, 0xBB8 );
	pOutput2 .Write( "WriteX(0, 0, \"Level0_Categ0\", 0x28, 0x190, 0xFA0, 0x9C40 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x28, 0x190, 0xFA0, 0x9C40 );
	pOutput2 .Write( "WriteX(1, 1, \"Level1_Categ1\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x17, 0x080, 0xFA0, 0x8C40 );
	pOutput2 .Write( "WriteX(1, 2, \"Level1_Categ2\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(1, 2, "Level1_Categ2", 0x17, 0x080, 0xFA0, 0x8C40 );
	pOutput2 .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x17, 0x080, 0xFA0, 0x8C40 );
	i = 0xEFCDAB12;
	pOutput2 .Write( "i = 0xEFCDAB12");
	pOutput2 .Write( "Dump(0, 0, \"Level0_Categ0_intDump\", &i, 4)\n");
	testLog.Dump(0, 0, "Level0_Categ0_intDump", &i, 4);
	pOutput2 .Write( "Dump(1, 1, \"Level1_Categ1_intDump\", &i, 4)\n");
	testLog.Dump(1, 1, "Level1_Categ1_intDump", &i, 4);
	pOutput2 .Write( "Dump(1, 2, \"Level1_Categ2_intDump\", &i, 4)\n");
	testLog.Dump(1, 2, "Level1_Categ2_intDump", &i, 4);
	pOutput2 .Write( "Dump(7, 31, \"Level7_Categ31_intDump\", &i, 4)\n");
	testLog.Dump(7, 31, "Level7_Categ31_intDump", &i, 4);
	a = 'a';
	pOutput2 .Write( "a = 'a'");
	pOutput2 .Write( "Dump(0, 0, \"Level0_Categ0_charDump\", &a, 1)\n");
	testLog.Dump(0, 0, "Level0_Categ0_charDump", &a, 1);
	pOutput2 .Write( "Dump(1, 1, \"Level1_Categ1_charDump\", &a, 1)\n");
	testLog.Dump(1, 1, "Level1_Categ1_charDump", &a, 1);
	pOutput2 .Write( "Dump(1, 2, \"Level1_Categ2_charDump\", &a, 1)\n");
	testLog.Dump(1, 2, "Level1_Categ2_charDump", &a, 1);
	pOutput2 .Write( "Dump(7, 31, \"Level7_Categ31_charDump\", &a, 1)\n");
	testLog.Dump(7, 31, "Level7_Categ31_charDump", &a, 1);
	pafee_memcpy(str,"dump2",5);
	pOutput2 .Write( "str[] = \"dump2\"");
	pOutput2 .Write( "Dump(0, 0, \"Level0_Categ0_stringDump\", str, 5)\n");
	testLog.Dump(0, 0, "Level0_Categ0_stringDump", str, 5);
	pOutput2 .Write( "Dump(1, 1, \"Level1_Categ1_stringDump\", str, 5)\n");
	testLog.Dump(1, 1, "Level1_Categ1_stringDump", str, 5);
	pOutput2 .Write( "Dump(1, 2, \"Level1_Categ2_stringDump\", str, 5)\n");
	testLog.Dump(1, 2, "Level1_Categ2_stringDump", str, 5);
	pOutput2 .Write( "Dump(7, 31, \"Level7_Categ31_stringDump\", str, 5)\n");
	testLog.Dump(7, 31, "Level7_Categ31_stringDump", str, 5);
	//test pattern 3
	StdStreamOutput pOutput3;
	testLog.SetLogOutput(&pOutput3);
	testLog.SetLevelString(1, "LevelC");	
	testLog.SetCategoryString(2, "Cat2");	
	testLog.SetLogLevel(10);	
	pOutput3 .Write( "SetLevelString(1, \"LevelC\")\n");
	pOutput3 .Write( "SetCategoryString(2, \"Cat2\")\n");
	pOutput3 .Write( "testLog.SetLogLevel(10)\n");
	pOutput3 .Write( "Write(0, 0, \"Level0_Categ0\")\n");
	testLog.Write(0, 0, "Level0_Categ0");
	pOutput3 .Write( "Write(1, 1, \"Level1_Categ1\")\n");
	testLog.Write(1, 1, "Level1_Categ1");
	pOutput3 .Write( "Write(1, 2, \"Level1_Categ2\")\n");
	testLog.Write(1, 2, "Level1_Categ2");
	pOutput3 .Write( "Write(7, 2, \"Level7_Categ31\")\n");
	testLog.Write(7, 31, "Level7_Categ31");
	pOutput3 .Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest1A\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest1A");
	pOutput3 .Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest1A\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest1A");
	pOutput3 .Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest1A\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest1A");
	pOutput3 .Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest1A\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest1A");
	pOutput3 .Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest2A", "Logtest2B");
	pOutput3 .Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest2A", "Logtest2B");
	pOutput3 .Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest2A", "Logtest2B");
	pOutput3 .Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest2A", "Logtest2B");
	pOutput3 .Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput3 .Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput3 .Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput3 .Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput3 .Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput3 .Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput3 .Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput3 .Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput3 .Write( "Write(0, 0, \"Level0_Categ0\", 10)\n");
	testLog.Write(0, 0, "Level0_Categ0", 10);
	pOutput3 .Write( "Write(1, 1, \"Level1_Categ1\", 10 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 10 );
	pOutput3 .Write( "Write(1, 2, \"Level1_Categ2\", 10 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 10 );
	pOutput3 .Write( "Write(7, 31, \"Level7_Categ31\", 10 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 10 );
	pOutput3 .Write( "Write(0, 0, \"Level0_Categ0\", 20, 200 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 20, 200 );
	pOutput3 .Write( "Write(1, 1, \"Level1_Categ1\", 20, 200 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 20, 200 );
	pOutput3 .Write( "Write(1, 2, \"Level1_Categ2\", 20, 200 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 20, 200 );
	pOutput3 .Write( "Write(7, 31, \"Level7_Categ31\", 20, 200 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 20, 200 );
	pOutput3 .Write( "Write(0, 0, \"Level0_Categ0\", 30, 300, 3000 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 30, 300, 3000 );
	pOutput3 .Write( "Write(1, 1, \"Level1_Categ1\", 30, 300, 3000 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 30, 300, 3000 );
	pOutput3 .Write( "Write(1, 2, \"Level1_Categ2\", 30, 300, 3000 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 30, 300, 3000 );
	pOutput3 .Write( "Write(7, 31, \"Level7_Categ31\", 30, 300, 3000 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 30, 300, 3000 );
	pOutput3 .Write( "Write(0, 0, \"Level0_Categ0\", 40, 400, 4000, 40000 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 40, 400, 4000, 40000 );
	pOutput3 .Write( "Write(1, 1, \"Level1_Categ1\", 40, 400, 4000, 40000 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 40, 400, 4000, 40000 );
	pOutput3 .Write( "Write(1, 2, \"Level1_Categ2\", 40, 400, 4000, 40000 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 40, 400, 4000, 40000 );
	pOutput3 .Write( "Write(7, 31, \"Level7_Categ31\", 40, 400, 4000, 40000 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 40, 400, 4000, 40000 );
	pOutput3 .Write( "WriteX(0, 0, \"Level0_Categ0\", 0x14, 0xC8 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x14, 0xC8 );
	pOutput3 .Write( "WriteX(1, 1, \"Level1_Categ1\", 0x14, 0xC8 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x14, 0xC8 );
	pOutput3 .Write( "WriteX(1, 2, \"Level1_Categ2\", 0x14, 0xC8 )\n");
	testLog.WriteX(1, 2, "Level1_Categ2", 0x14, 0xC8 );
	pOutput3 .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x14, 0xC8 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x14, 0xC8 );
	pOutput3 .Write( "WriteX(0, 0, \"Level0_Categ0\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x1E, 0x12C, 0xBB8 );
	pOutput3 .Write( "WriteX(1, 1, \"Level1_Categ1\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x1E, 0x12C, 0xBB8 );
	pOutput3 .Write( "WriteX(1, 2, \"Level1_Categ2\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(1, 2, "Level1_Categ2", 0x1E, 0x12C, 0xBB8 );
	pOutput3 .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x1E, 0x12C, 0xBB8 );
	pOutput3 .Write( "WriteX(0, 0, \"Level0_Categ0\", 0x28, 0x190, 0xFA0, 0x9C40 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x28, 0x190, 0xFA0, 0x9C40 );
	pOutput3 .Write( "WriteX(1, 1, \"Level1_Categ1\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x17, 0x080, 0xFA0, 0x8C40 );
	pOutput3 .Write( "WriteX(1, 2, \"Level1_Categ2\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(1, 2, "Level1_Categ2", 0x17, 0x080, 0xFA0, 0x8C40 );
	pOutput3 .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x17, 0x080, 0xFA0, 0x8C40 );
	i = 0xEFCDAB12;
	pOutput3 .Write( "i = 0xEFCDAB12");
	pOutput3 .Write( "Dump(0, 0, \"Level0_Categ0_intDump\", &i, 4)\n");
	testLog.Dump(0, 0, "Level0_Categ0_intDump", &i, 4);
	pOutput3 .Write( "Dump(1, 1, \"Level1_Categ1_intDump\", &i, 4)\n");
	testLog.Dump(1, 1, "Level1_Categ1_intDump", &i, 4);
	pOutput3 .Write( "Dump(1, 2, \"Level1_Categ2_intDump\", &i, 4)\n");
	testLog.Dump(1, 2, "Level1_Categ2_intDump", &i, 4);
	pOutput3 .Write( "Dump(7, 31, \"Level7_Categ31_intDump\", &i, 4)\n");
	testLog.Dump(7, 31, "Level7_Categ31_intDump", &i, 4);
	a = 'a';
	pOutput3 .Write( "a = 'a'");
	pOutput3 .Write( "Dump(0, 0, \"Level0_Categ0_charDump\", &a, 1)\n");
	testLog.Dump(0, 0, "Level0_Categ0_charDump", &a, 1);
	pOutput3 .Write( "Dump(1, 1, \"Level1_Categ1_charDump\", &a, 1)\n");
	testLog.Dump(1, 1, "Level1_Categ1_charDump", &a, 1);
	pOutput3 .Write( "Dump(1, 2, \"Level1_Categ2_charDump\", &a, 1)\n");
	testLog.Dump(1, 2, "Level1_Categ2_charDump", &a, 1);
	pOutput3 .Write( "Dump(7, 31, \"Level7_Categ31_charDump\", &a, 1)\n");
	testLog.Dump(7, 31, "Level7_Categ31_charDump", &a, 1);
	pafee_memcpy(str,"dump2",5);
	pOutput3 .Write( "str[] = \"dump2\"");
	pOutput3 .Write( "Dump(0, 0, \"Level0_Categ0_stringDump\", str, 5)\n");
	testLog.Dump(0, 0, "Level0_Categ0_stringDump", str, 5);
	pOutput3 .Write( "Dump(1, 1, \"Level1_Categ1_stringDump\", str, 5)\n");
	testLog.Dump(1, 1, "Level1_Categ1_stringDump", str, 5);
	pOutput3 .Write( "Dump(1, 2, \"Level1_Categ2_stringDump\", str, 5)\n");
	testLog.Dump(1, 2, "Level1_Categ2_stringDump", str, 5);
	pOutput3 .Write( "Dump(7, 31, \"Level7_Categ31_stringDump\", str, 5)\n");
	testLog.Dump(7, 31, "Level7_Categ31_stringDump", str, 5);
	//test pattern 4
	StdStreamOutput pOutput4;
	testLog.SetLogOutput(&pOutput4);
	testLog.SetLevelString(7, "Level7");	
	testLog.SetCategoryString(31, "Cat31");	
	testLog.SetLogLevel(7);
	pOutput4 .Write( "SetLevelString(7, \"Level7\")\n");
	pOutput4 .Write( "SetCategoryString(31, \"Cat31\")\n");
	pOutput4 .Write( "SetLogLevel(7)\n");
	pOutput4 .Write( "Write(0, 0, \"Level0_Categ0\")\n");
	testLog.Write(0, 0, "Level0_Categ0");
	pOutput4 .Write( "Write(1, 1, \"Level1_Categ1\")\n");
	testLog.Write(1, 1, "Level1_Categ1");
	pOutput4 .Write( "Write(1, 2, \"Level1_Categ2\")\n");
	testLog.Write(1, 2, "Level1_Categ2");
	pOutput4 .Write( "Write(7, 31, \"Level7_Categ31\")\n");
	testLog.Write(7, 31, "Level7_Categ31");
	pOutput4 .Write( "Write(8, 31, \"Level8_Categ31\")\n");
	testLog.Write(8, 31, "Level7_Categ31");
	pOutput4 .Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest1A\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest1A");
	pOutput4 .Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest1A\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest1A");
	pOutput4 .Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest1A\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest1A");
	pOutput4 .Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest1A\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest1A");
	pOutput4 .Write( "Write(8, 31, \"Level8_Categ31\", \"Logtest1A\")\n");
	testLog.Write(8, 31, "Level8_Categ31", "Logtest1A");
	pOutput4 .Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest2A", "Logtest2B");
	pOutput4 .Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest2A", "Logtest2B");
	pOutput4 .Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest2A", "Logtest2B");
	pOutput4 .Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest2A", "Logtest2B");
	pOutput4 .Write( "Write(8, 31, \"Level8_Categ31\", \"Logtest2A\", \"Logtest2B\")\n");
	testLog.Write(8, 31, "Level8_Categ31", "Logtest2A", "Logtest2B");
	pOutput4 .Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput4 .Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput4 .Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput4 .Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput4 .Write( "Write(8, 31, \"Level8_Categ31\", \"Logtest3A\", \"Logtest3B\", \"Logtest3C\")\n");
	testLog.Write(8, 31, "Level8_Categ31", "Logtest3A", "Logtest3B", "Logtest3C");
	pOutput4 .Write( "Write(0, 0, \"Level0_Categ0\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(0, 0, "Level0_Categ0", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput4 .Write( "Write(1, 1, \"Level1_Categ1\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(1, 1, "Level1_Categ1", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput4 .Write( "Write(1, 2, \"Level1_Categ2\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(1, 2, "Level1_Categ2", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput4 .Write( "Write(7, 31, \"Level7_Categ31\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(7, 31, "Level7_Categ31", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput4 .Write( "Write(8, 31, \"Level8_Categ31\", \"Logtest4A\", \"Logtest4B\", \"Logtest4C\",\"Logtest4D\")\n");
	testLog.Write(8, 31, "Level8_Categ31", "Logtest4A", "Logtest4B", "Logtest4C","Logtest4D");
	pOutput4 .Write( "Write(0, 0, \"Level0_Categ0\", 10)\n");
	testLog.Write(0, 0, "Level0_Categ0", 10);
	pOutput4 .Write( "Write(1, 1, \"Level1_Categ1\", 10 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 10 );
	pOutput4 .Write( "Write(1, 2, \"Level1_Categ2\", 10 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 10 );
	pOutput4 .Write( "Write(7, 31, \"Level7_Categ31\", 10 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 10 );
	pOutput4 .Write( "Write(8, 31, \"Level8_Categ31\", 10 )\n");
	testLog.Write(8, 31, "Level8_Categ31", 10 );
	pOutput4 .Write( "Write(0, 0, \"Level0_Categ0\", 20, 200 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 20, 200 );
	pOutput4 .Write( "Write(1, 1, \"Level1_Categ1\", 20, 200 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 20, 200 );
	pOutput4 .Write( "Write(1, 2, \"Level1_Categ2\", 20, 200 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 20, 200 );
	pOutput4 .Write( "Write(7, 31, \"Level7_Categ31\", 20, 200 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 20, 200 );
	pOutput4 .Write( "Write(8, 31, \"Level8_Categ31\", 20, 200 )\n");
	testLog.Write(8, 31, "Level8_Categ31", 20, 200 );
	pOutput4 .Write( "Write(0, 0, \"Level0_Categ0\", 30, 300, 3000 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 30, 300, 3000 );
	pOutput4 .Write( "Write(1, 1, \"Level1_Categ1\", 30, 300, 3000 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 30, 300, 3000 );
	pOutput4 .Write( "Write(1, 2, \"Level1_Categ2\", 30, 300, 3000 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 30, 300, 3000 );
	pOutput4 .Write( "Write(7, 31, \"Level7_Categ31\", 30, 300, 3000 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 30, 300, 3000 );
	pOutput4 .Write( "Write(8, 31, \"Level8_Categ31\", 30, 300, 3000 )\n");
	testLog.Write(8, 31, "Level8_Categ31", 30, 300, 3000 );
	pOutput4 .Write( "Write(0, 0, \"Level0_Categ0\", 40, 400, 4000, 40000 )\n");
	testLog.Write(0, 0, "Level0_Categ0", 40, 400, 4000, 40000 );
	pOutput4 .Write( "Write(1, 1, \"Level1_Categ1\", 40, 400, 4000, 40000 )\n");
	testLog.Write(1, 1, "Level1_Categ1", 40, 400, 4000, 40000 );
	pOutput4 .Write( "Write(1, 2, \"Level1_Categ2\", 40, 400, 4000, 40000 )\n");
	testLog.Write(1, 2, "Level1_Categ2", 40, 400, 4000, 40000 );
	pOutput4 .Write( "Write(7, 31, \"Level7_Categ31\", 40, 400, 4000, 40000 )\n");
	testLog.Write(7, 31, "Level7_Categ31", 40, 400, 4000, 40000 );
	pOutput4 .Write( "Write(8, 31, \"Level8_Categ31\", 40, 400, 4000, 40000 )\n");
	testLog.Write(8, 31, "Level8_Categ31", 40, 400, 4000, 40000 );
	pOutput4 .Write( "WriteX(0, 0, \"Level0_Categ0\", 0x14, 0xC8 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x14, 0xC8 );
	pOutput4 .Write( "WriteX(1, 1, \"Level1_Categ1\", 0x14, 0xC8 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x14, 0xC8 );
	pOutput4 .Write( "WriteX(1, 2, \"Level1_Categ2\", 0x14, 0xC8 )\n");
	testLog.WriteX(1, 2, "Level1_Categ2", 0x14, 0xC8 );
	pOutput4 .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x14, 0xC8 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x14, 0xC8 );
	pOutput4 .Write( "WriteX(8, 31, \"Level8_Categ31\", 0x14, 0xC8 )\n");
	testLog.WriteX(8, 31, "Level8_Categ31", 0x14, 0xC8 );
	pOutput4 .Write( "WriteX(0, 0, \"Level0_Categ0\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x1E, 0x12C, 0xBB8 );
	pOutput4 .Write( "WriteX(1, 1, \"Level1_Categ1\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x1E, 0x12C, 0xBB8 );
	pOutput4 .Write( "WriteX(1, 2, \"Level1_Categ2\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(1, 2, "Level1_Categ2", 0x1E, 0x12C, 0xBB8 );
	pOutput4 .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x1E, 0x12C, 0xBB8 );
	pOutput4 .Write( "WriteX(8, 31, \"Level8_Categ31\", 0x1E, 0x12C, 0xBB8 )\n");
	testLog.WriteX(8, 31, "Level8_Categ31", 0x1E, 0x12C, 0xBB8 );
	pOutput4 .Write( "WriteX(0, 0, \"Level0_Categ0\", 0x28, 0x190, 0xFA0, 0x9C40 )\n");
	testLog.WriteX(0, 0, "Level0_Categ0", 0x28, 0x190, 0xFA0, 0x9C40 );
	pOutput4 .Write( "WriteX(1, 1, \"Level1_Categ1\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(1, 1, "Level1_Categ1", 0x17, 0x080, 0xFA0, 0x8C40 );
	pOutput4 .Write( "WriteX(1, 2, \"Level1_Categ2\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(1, 2, "Level1_Categ2", 0x17, 0x080, 0xFA0, 0x8C40 );
	pOutput4 .Write( "WriteX(7, 31, \"Level7_Categ31\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(7, 31, "Level7_Categ31", 0x17, 0x080, 0xFA0, 0x8C40 );
	pOutput4 .Write( "WriteX(8, 31, \"Level8_Categ31\", 0x17, 0x080, 0xFA0, 0x8C40 )\n");
	testLog.WriteX(8, 31, "Level8_Categ31", 0x17, 0x080, 0xFA0, 0x8C40 );
	i = 0xEFCDAB12;
	pOutput4 .Write( "i = 0xEFCDAB12");
	pOutput4 .Write( "Dump(0, 0, \"Level0_Categ0_intDump\", &i, 4)\n");
	testLog.Dump(0, 0, "Level0_Categ0_intDump", &i, 4);
	pOutput4 .Write( "Dump(1, 1, \"Level1_Categ1_intDump\", &i, 4)\n");
	testLog.Dump(1, 1, "Level1_Categ1_intDump", &i, 4);
	pOutput4 .Write( "Dump(1, 2, \"Level1_Categ2_intDump\", &i, 4)\n");
	testLog.Dump(1, 2, "Level1_Categ2_intDump", &i, 4);
	pOutput4 .Write( "Dump(7, 31, \"Level7_Categ31_intDump\", &i, 4)\n");
	testLog.Dump(7, 31, "Level7_Categ31_intDump", &i, 4);
	pOutput4 .Write( "Dump(8, 31, \"Level8_Categ31_intDump\", &i, 4)\n");
	testLog.Dump(8, 31, "Level8_Categ31_intDump", &i, 4);
	a = 'a';
	pOutput4 .Write( "a = 'a'");
	pOutput4 .Write( "Dump(0, 0, \"Level0_Categ0_charDump\", &a, 1)\n");
	testLog.Dump(0, 0, "Level0_Categ0_charDump", &a, 1);
	pOutput4 .Write( "Dump(1, 1, \"Level1_Categ1_charDump\", &a, 1)\n");
	testLog.Dump(1, 1, "Level1_Categ1_charDump", &a, 1);
	pOutput4 .Write( "Dump(1, 2, \"Level1_Categ2_charDump\", &a, 1)\n");
	testLog.Dump(1, 2, "Level1_Categ2_charDump", &a, 1);
	pOutput4 .Write( "Dump(7, 31, \"Level7_Categ31_charDump\", &a, 1)\n");
	testLog.Dump(7, 31, "Level7_Categ31_charDump", &a, 1);
	pOutput4 .Write( "Dump(8, 31, \"Level8_Categ31_charDump\", &a, 1)\n");
	testLog.Dump(8, 31, "Level8_Categ31_charDump", &a, 1);
	pafee_memcpy(str,"dump2",5);
	pOutput4 .Write( "str[] = \"dump2\"");
	pOutput4 .Write( "Dump(0, 0, \"Level0_Categ0_stringDump\", str, 5)\n");
	testLog.Dump(0, 0, "Level0_Categ0_stringDump", str, 5);
	pOutput4 .Write( "Dump(1, 1, \"Level1_Categ1_stringDump\", str, 5)\n");
	testLog.Dump(1, 1, "Level1_Categ1_stringDump", str, 5);
	pOutput4 .Write( "Dump(1, 2, \"Level1_Categ2_stringDump\", str, 5)\n");
	testLog.Dump(1, 2, "Level1_Categ2_stringDump", str, 5);
	pOutput4 .Write( "Dump(7, 31, \"Level7_Categ31_stringDump\", str, 5)\n");
	testLog.Dump(7, 31, "Level7_Categ31_stringDump", str, 5);
	pOutput4 .Write( "Dump(8, 31, \"Level8_Categ31_stringDump\", str, 5)\n");
	testLog.Dump(8, 31, "Level8_Categ31_stringDump", str, 5);
}

#endif // PAFEE_UNITTEST

}; // Pafee

