//****************************************************
//
//  project   PAFEE
//! @file:    PafeeLog.h
//! @author:  Pengkai Cheng
//! @brief:   Log management class
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//


#ifndef _LOG_H_INCLUDED_
#define _LOG_H_INCLUDED_

#include "pafee_stdio.h"
#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "pafee_string.h"
#include "PafeeStreamOutput.h"
#ifdef PAFEE_UNITTEST
#include "PafeeStdStreamOutput.h"
#endif
#include <fstream>

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif //PAFEE_UNITTEST

using   namespace   std; 

namespace Pafee
{

//------------------------------------------------------------------------------
//!    - @brief    General log output function ; 
//!    - @brief    Only support fixed parameter
//!        - Log could be output
//!        - At most eight log levels can be set
//!        - At most 32 category levels can be set
//!        - Output format image
//!        - [level][category] log string (P1,P2,P3,P4)
//!        - The log output can be controlled according to the log level
//!    .
//------------------------------------------------------------------------------
//! Log
class Log : public Object
{

public:
	Log(StreamOutput* pOutput=NULL);
	~Log();
	void Write(int level, int category, const char* stringLog);
	void Write(int level, int category, const char* stringLog, char* p1, char* p2=NULL, char* p3=NULL, char* p4=NULL);
	void Write(int level, int category, const char* stringLog, int n1);
	void Write(int level, int category, const char* stringLog, int n1, int n2);
	void Write(int level, int category, const char* stringLog, int n1, int n2, int n3);
	void Write(int level, int category, const char* stringLog, int n1, int n2, int n3, int n4);
	void WriteX(int level, int category, const char* stringLog, int n1, int n2);
	void WriteX(int level, int category, const char* stringLog, int n1, int n2, int n3);
	void WriteX(int level, int category, const char* stringLog, int n1, int n2, int n3, int n4);
	void Dump(int level, int category, const char* stringLog, void* pData, int length);
	void SetLevelString(int level, char* levelString);
	void SetCategoryString(int category, char* categoryString);
	void SetLogLevel(int level);
	void SetLogOutput(StreamOutput* pOutput);

private:
	StreamOutput* m_pOutput;
	int m_level;
	char* m_pLevelString[8];
	int m_category;
	char* m_pCategoryString[32];
	const char* m_pStringLog;
	char* m_p1;
	char* m_p2;
	char* m_p3;
	char* m_p4;
	int m_n1;
	int m_n2;
	int m_n3;
	int m_n4;
	unsigned char* m_pData;
	int m_length;
	void BasicWrite(int level, int category, const char* stringLog);
	void IntegerWrite(int n);
	void HexWrite(int n);

#ifdef PAFEE_UNITTEST
public: // unit test function
	static void UnitTest();
	static void UnitTest(StreamOutput* pOutput);
#endif // PAFEE_UNITTEST
};

};  // Pafee

#endif