#include "PafeeQueueT.h"
#include <string>

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif
#include "PafeeString.h"

using namespace Pafee;
using namespace std;

int main()
{
	{
		const int TEST_QUEUE_COUNT = 10;
		int i;
		int count;

		QueueT<string> queue;

		string str = "1234567890";
		string str2 = "abcdefghijklmn";
		string strOut;

		//STATISTICS
		int addCount=0;
		int maxElementCount=0;
		int presentElementCount=0;
#ifdef PAFEE_STATISTICS_ENABLE
		QueueT<string>::STATISTICS statisticsRtn;
#endif
		//
		//string
		//
		{
			// test GetCount/IsEmpty
			assert(queue.GetCount() == 0);
			assert(queue.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// Enqueue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue.Enqueue(str);
				addCount++;
				presentElementCount++;
			}
			maxElementCount = presentElementCount;
			assert(queue.GetCount() == presentElementCount);
			assert(!queue.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// Dequeue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue.Dequeue(strOut);
				presentElementCount--;
			}
			assert(queue.GetCount() == presentElementCount);
			assert(queue.IsEmpty());
			assert(!queue.Dequeue(strOut));
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// Enqueue/Dequeue/Enqueue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue.Enqueue(str);
				addCount++;
				presentElementCount++;
			}
			assert(queue.GetCount() == presentElementCount);
			assert(!queue.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
			for(i=0; i<TEST_QUEUE_COUNT/2; i++)
			{
				queue.Dequeue(strOut);
				presentElementCount--;
			}
			assert(queue.GetCount() == presentElementCount);
			assert(!queue.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue.Enqueue(str2);
				addCount++;
				presentElementCount++;
				if(maxElementCount<presentElementCount)
				{
					maxElementCount = presentElementCount;
				}
			}
			assert(queue.GetCount() == presentElementCount);
			assert(!queue.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// test Peek
			int maxCount = presentElementCount;
			for(i=0;i<maxCount;i++)
			{
				strOut = queue.Peek();
				if(i<(TEST_QUEUE_COUNT/2))
				{
					assert(str == strOut);
				}
				else
				{
					assert(str2 == strOut);
				}
				queue.RemoveFront();
				presentElementCount--;
			}
			assert(!queue.Dequeue(strOut));
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveFront
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue.Enqueue(str);
				addCount++;
				presentElementCount++;
			}
			//RemoveFront
			queue.RemoveFront();
			presentElementCount--;
			count = queue.GetCount();
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveFront
			queue.RemoveFront();
			presentElementCount--;
			count = queue.GetCount();
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveAll
			queue.RemoveAll();
			count = queue.GetCount();
			presentElementCount=0;
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		}


		//
		//PafeeString
		//
		QueueT<Pafee::String> queue1;

		String pafeeStr = "1234567890";
		String pafeeStr2 = "abcdefghijklmn";
		String pafeeStrOut;

		//STATISTICS
		addCount=0;
		maxElementCount=0;
		presentElementCount=0;
#ifdef PAFEE_STATISTICS_ENABLE
		QueueT<String>::STATISTICS pafeeStatisticsRtn;
#endif


		{
			// test GetCount/IsEmpty
			assert(queue1.GetCount() == 0);
			assert(queue1.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// Enqueue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1.Enqueue(pafeeStr);
				addCount++;
				presentElementCount++;
			}
			maxElementCount = presentElementCount;
			assert(queue1.GetCount() == presentElementCount);
			assert(!queue1.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// Dequeue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1.Dequeue(pafeeStrOut);
				presentElementCount--;
			}
			assert(queue1.GetCount() == presentElementCount);
			assert(queue1.IsEmpty());
			assert(!queue1.Dequeue(pafeeStrOut));
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// Enqueue/Dequeue/Enqueue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1.Enqueue(pafeeStr);
				addCount++;
				presentElementCount++;
			}
			assert(queue1.GetCount() == presentElementCount);
			assert(!queue1.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif
			for(i=0; i<TEST_QUEUE_COUNT/2; i++)
			{
				queue1.Dequeue(pafeeStrOut);
				presentElementCount--;
			}
			assert(queue1.GetCount() == presentElementCount);
			assert(!queue1.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1.Enqueue(pafeeStr2);
				addCount++;
				presentElementCount++;
				if(maxElementCount<presentElementCount)
				{
					maxElementCount = presentElementCount;
				}
			}
			assert(queue1.GetCount() == presentElementCount);
			assert(!queue1.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// test Peek
			int maxCount = presentElementCount;
			for(i=0;i<maxCount;i++)
			{
				pafeeStrOut = queue1.Peek();
				if(i<(TEST_QUEUE_COUNT/2))
				{
					assert(pafeeStr == pafeeStrOut);
				}
				else
				{
					assert(pafeeStr2 == pafeeStrOut);
				}
				queue1.RemoveFront();
				presentElementCount--;
			}
			assert(!queue1.Dequeue(pafeeStrOut));
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveFront
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1.Enqueue(pafeeStr);
				addCount++;
				presentElementCount++;
			}
			//RemoveFront
			queue1.RemoveFront();
			presentElementCount--;
			count = queue1.GetCount();
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveFront
			queue1.RemoveFront();
			presentElementCount--;
			count = queue1.GetCount();
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveAll
			queue1.RemoveAll();
			count = queue1.GetCount();
			presentElementCount=0;
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif
		}
	}
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
	{
		const int TEST_QUEUE_COUNT = 10;
		int i;
		int count;

		QueueT<string> queuet0c;

		string str = "1234567890";
		string str2 = "abcdefghijklmn";
		string strOut;

		//STATISTICS
		int addCount=0;
		int maxElementCount=0;
		int presentElementCount=0;
#ifdef PAFEE_STATISTICS_ENABLE
		QueueT<string>::STATISTICS statisticsRtn;
#endif
		//
		//string
		//
		{
			// test GetCount/IsEmpty
			assert(queuet0c.GetCount() == 0);
			assert(queuet0c.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queuet0c.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// Enqueue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queuet0c.Enqueue(str);
				addCount++;
				presentElementCount++;
			}
			maxElementCount = presentElementCount;
			assert(queuet0c.GetCount() == presentElementCount);
			assert(!queuet0c.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queuet0c.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
			
			//QueueT<string> queue = queuet0c;
			QueueT<string> queue;
			queue = queuet0c;

			// Dequeue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue.Dequeue(strOut);
				presentElementCount--;
			}
			assert(queue.GetCount() == presentElementCount);
			assert(queue.IsEmpty());
			assert(!queue.Dequeue(strOut));
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// Enqueue/Dequeue/Enqueue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue.Enqueue(str);
				addCount++;
				presentElementCount++;
			}
			assert(queue.GetCount() == presentElementCount);
			assert(!queue.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
			for(i=0; i<TEST_QUEUE_COUNT/2; i++)
			{
				queue.Dequeue(strOut);
				presentElementCount--;
			}
			assert(queue.GetCount() == presentElementCount);
			assert(!queue.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue.Enqueue(str2);
				addCount++;
				presentElementCount++;
				if(maxElementCount<presentElementCount)
				{
					maxElementCount = presentElementCount;
				}
			}
			assert(queue.GetCount() == presentElementCount);
			assert(!queue.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// test Peek
			int maxCount = presentElementCount;
			for(i=0;i<maxCount;i++)
			{
				strOut = queue.Peek();
				if(i<(TEST_QUEUE_COUNT/2))
				{
					assert(str == strOut);
				}
				else
				{
					assert(str2 == strOut);
				}
				queue.RemoveFront();
				presentElementCount--;
			}
			assert(!queue.Dequeue(strOut));
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveFront
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue.Enqueue(str);
				addCount++;
				presentElementCount++;
			}
			//RemoveFront
			queue.RemoveFront();
			presentElementCount--;
			count = queue.GetCount();
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveFront
			queue.RemoveFront();
			presentElementCount--;
			count = queue.GetCount();
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveAll
			queue.RemoveAll();
			count = queue.GetCount();
			presentElementCount=0;
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.addCount == addCount);
			assert(statisticsRtn.maxElementCount == maxElementCount);
			assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		}


		//
		//PafeeString
		//
		QueueT<String> queue1x0c;

		String pafeeStr = "1234567890";
		String pafeeStr2 = "abcdefghijklmn";
		String pafeeStrOut;

		//STATISTICS
		addCount=0;
		maxElementCount=0;
		presentElementCount=0;
#ifdef PAFEE_STATISTICS_ENABLE
		QueueT<String>::STATISTICS pafeeStatisticsRtn;
#endif


		{
			// test GetCount/IsEmpty
			assert(queue1x0c.GetCount() == 0);
			assert(queue1x0c.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1x0c.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// Enqueue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1x0c.Enqueue(pafeeStr);
				addCount++;
				presentElementCount++;
			}
			maxElementCount = presentElementCount;
			assert(queue1x0c.GetCount() == presentElementCount);
			assert(!queue1x0c.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1x0c.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			QueueT<String> queue1 =queue1x0c;





			// Dequeue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1.Dequeue(pafeeStrOut);
				presentElementCount--;
			}
			assert(queue1.GetCount() == presentElementCount);
			assert(queue1.IsEmpty());
			assert(!queue1.Dequeue(pafeeStrOut));
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// Enqueue/Dequeue/Enqueue
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1.Enqueue(pafeeStr);
				addCount++;
				presentElementCount++;
			}
			assert(queue1.GetCount() == presentElementCount);
			assert(!queue1.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif
			for(i=0; i<TEST_QUEUE_COUNT/2; i++)
			{
				queue1.Dequeue(pafeeStrOut);
				presentElementCount--;
			}
			assert(queue1.GetCount() == presentElementCount);
			assert(!queue1.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1.Enqueue(pafeeStr2);
				addCount++;
				presentElementCount++;
				if(maxElementCount<presentElementCount)
				{
					maxElementCount = presentElementCount;
				}
			}
			assert(queue1.GetCount() == presentElementCount);
			assert(!queue1.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// test Peek
			int maxCount = presentElementCount;
			for(i=0;i<maxCount;i++)
			{
				pafeeStrOut = queue1.Peek();
				if(i<(TEST_QUEUE_COUNT/2))
				{
					assert(pafeeStr == pafeeStrOut);
				}
				else
				{
					assert(pafeeStr2 == pafeeStrOut);
				}
				queue1.RemoveFront();
				presentElementCount--;
			}
			assert(!queue1.Dequeue(pafeeStrOut));
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveFront
			for(i=0; i<TEST_QUEUE_COUNT; i++)
			{
				queue1.Enqueue(pafeeStr);
				addCount++;
				presentElementCount++;
			}
			//RemoveFront
			queue1.RemoveFront();
			presentElementCount--;
			count = queue1.GetCount();
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveFront
			queue1.RemoveFront();
			presentElementCount--;
			count = queue1.GetCount();
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif

			// test RemoveAll
			queue1.RemoveAll();
			count = queue1.GetCount();
			presentElementCount=0;
			assert(count == presentElementCount);
#ifdef PAFEE_STATISTICS_ENABLE
			queue1.GetStatistics(&pafeeStatisticsRtn);
			assert(pafeeStatisticsRtn.addCount == addCount);
			assert(pafeeStatisticsRtn.maxElementCount == maxElementCount);
			assert(pafeeStatisticsRtn.presentElementCount == presentElementCount);
#endif
		}
	}

	return 0;
}
