#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <unistd.h>
#include "PafeeRingBufferedListener.h"

using namespace Pafee;

//
// Listener
//
class MyListener : public RingBufferedListener
{
public:
	MyListener()
	{
		m_nextSN[0] = 0;
		m_nextSN[1] = 0;
		m_nextSN[2] = 0;
		m_isDone[0] = false;
		m_isDone[1] = false;
		m_isDone[2] = false;
	};
	~MyListener()
	{
	};
	
	void ShowResult(void)
	{
		printf("Receive from #1: %ld\n", m_nextSN[0]);
		printf("Receive from #2: %ld\n", m_nextSN[1]);
		printf("Receive from #3: %ld\n", m_nextSN[2]);
	};
	
	bool IsDone(void)
	{
		if(m_isDone[0] && m_isDone[1] && m_isDone[2])
		{
			return true;
		}
		return false;
	};

protected:
	virtual bool OnEventReceived(
							unsigned long eventID,///< [in] event id
							const void* pParam,	///< [in] parameter buffer
							int paramLength		///< [in] parameter length
							)
	{
		char szParam[256] = {0};
		unsigned long serialNo = 0x0000FFFF & eventID;
		unsigned long senderID = eventID >> 16;
		int index = senderID-1;

		memcpy(szParam, pParam, paramLength);

		// check serial number
		if(m_nextSN[index] == serialNo)
		{
			printf("Received event:%lX param:%s SN:%ld\n", eventID, szParam, serialNo);
			m_nextSN[index]++;
		}
		// end mark?
		else if(serialNo == 0xEEEE)
		{
			m_isDone[index] = true;
		}
		// lost
		else
		{
			printf("#### Invalid SN:%ld expect:%ld sender:%lX ####\n", serialNo, m_nextSN[index], senderID);
			exit(-1);
		}

		return true;
	};
	unsigned long m_nextSN[3];
	bool m_isDone[3];
};

//
// Sender
//
class EventSender : public PafeeTask
{
public:
	EventSender(int senderID = 0, Listener* pListener=NULL)
	{
		m_senderID = senderID;
		m_pListener = pListener;
	};

protected:

	virtual void* TaskProcedure(void* pParam)
	{
		char szParam[256];
		unsigned long eventID;
		for(int i = 0; i < 1000; i++)
		{
			eventID = (m_senderID << 16) | i;
			sprintf(szParam, "event param#%lX", eventID);
			m_pListener->SendEvent(eventID, szParam, strlen(szParam));
		}
		// Send end mark
		m_pListener->SendEvent((m_senderID << 16) | 0xEEEE, NULL, 0);
		return NULL;
	};

	int m_senderID;
	Listener* m_pListener;
};

//
// Test main
//
int main(int argc, void* argv[])
{
	MyListener listener;
	EventSender sender1(1, &listener);
	EventSender sender2(2, &listener);
	EventSender sender3(3, &listener);
	TASK_ID id1, id2, id3;

	//---------------------------------------------------------------
	// Three tasks send 1000 events to the listener.
	// the listener check the serial number of event and count it.
	//---------------------------------------------------------------

	listener.Initialize(3000, 256, (PafeeTask::TASK_PRIORITY) 50);
	sleep(1);

	sender1.Create(&id1, (PafeeTask::TASK_PRIORITY) 50);

	sender2.Create(&id2, (PafeeTask::TASK_PRIORITY) 50);

	sender3.Create(&id3, (PafeeTask::TASK_PRIORITY) 50);

	while(1)
	{
		if(listener.IsDone())
		{
			listener.ShowResult();
			break;
		}
		sleep(1);
	}
	printf("done,\n");
	return 0;
}

