#include <t_services.h>
#include <string.h>
#include <assert.h>
#include "PafeeEventDistributor.h"

void eventDistributorTest(void);


//
// Listener
//
class MyListener : public Pafee::Listener
{
public:
	MyListener(int listenerID = 0)
	{
		m_listenerID = listenerID;
	};

	~MyListener()
	{
	};

protected:
	virtual bool OnEventReceived(
							unsigned long eventID,	///< [in] event id
							const void* pParam,		///< [in] parameter buffer
							int paramLength			///< [in] parameter length
							)
	{
		char szParam[256] = {0};
		memcpy(szParam, pParam, paramLength);
		syslog(LOG_EMERG,"Listener#%d  received event:%lX  %s\n", m_listenerID, eventID, szParam);
		return true;
	};

	int m_listenerID;

};


//
// Test main
//
void eventDistributorTest(void)
{
	MyListener listener1(1);
	MyListener listener2(2);
	MyListener listener3(3);
	Pafee::EventDistributor distributor;

	assert(distributor.AddListener(&listener1));
	distributor.SendEvent(1, "param1", strlen("param1"));

	assert(distributor.AddListener(&listener2));
	distributor.SendEvent(2, "param2", strlen("param2"));

	assert(distributor.AddListener(&listener3));
	distributor.SendEvent(3, "param3", strlen("param3"));


	assert(distributor.RemoveListener(&listener1));
	distributor.SendEvent(4, "param4", strlen("param4"));

	assert(distributor.RemoveListener(&listener2));
	distributor.SendEvent(5, "param5", strlen("param5"));

	assert(distributor.RemoveListener(&listener3));
	distributor.SendEvent(6, "param6", strlen("param6"));


	syslog(LOG_EMERG,"--- send 10 listener ---");
	{
		int i;
		MyListener* listeners[11];

		for(i = 0; i < 11; i++)
		{
			listeners[i] = new MyListener(i);
			if(i == 10)
			{
				assert(distributor.AddListener(listeners[i]) == false);
			}
			else
			{
				assert(distributor.AddListener(listeners[i]));
			}
		}
		distributor.SendEvent(7, "param7", strlen("param7"));
		for(i = 0; i < 11; i++)
		{
			if(i == 10)
			{
				assert(distributor.RemoveListener(listeners[i]) == false);
			}
			else
			{
				assert(distributor.RemoveListener(listeners[i]));
			}
			delete listeners[i];
		}
		distributor.SendEvent(8, "param8", strlen("param8"));
	}
	syslog(LOG_EMERG,"done,");
}

