#include <t_services.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "kernel_id.h"
#include "PafeeRingBufferedListener.h"

using namespace Pafee;

//
// Listener
//
class MyRingBufferedListener : public RingBufferedListener
{
public:
	MyRingBufferedListener()
	{
		m_nextSN[0] = 0;
		m_nextSN[1] = 0;
		m_nextSN[2] = 0;
		m_isDone[0] = false;
		m_isDone[1] = false;
		m_isDone[2] = false;
	};
	~MyRingBufferedListener()
	{
	};
	
	void ShowResult(void)
	{
		syslog(LOG_EMERG,"Receive from #1: %ld", m_nextSN[0]);
		syslog(LOG_EMERG,"Receive from #2: %ld", m_nextSN[1]);
		syslog(LOG_EMERG,"Receive from #3: %ld", m_nextSN[2]);
	};
	
	bool IsDone(void)
	{
		if(m_isDone[0] && m_isDone[1] && m_isDone[2])
		{
			return true;
		}
		return false;
	};

protected:
	virtual bool OnEventReceived(
							unsigned long eventID,///< [in] event id
							const void* pParam,	///< [in] parameter buffer
							int paramLength		///< [in] parameter length
							)
	{
		char szParam[256] = {0};
		unsigned long serialNo = 0x0000FFFF & eventID;
		unsigned long senderID = eventID >> 16;
		int index = senderID-1;

		memcpy(szParam, pParam, paramLength);

		// check serial number
		if(m_nextSN[index] == serialNo)
		{
			syslog(LOG_EMERG,"Received event:%lX param:%s SN:%ld", eventID, szParam, serialNo);
			m_nextSN[index]++;
		}
		// end mark?
		else if(serialNo == 0xEEEE)
		{
			m_isDone[index] = true;
		}
		// lost
		else
		{
			syslog(LOG_EMERG,"#### Invalid SN:%ld expect:%ld sender:%lX ####", serialNo, m_nextSN[index], senderID);
			exit(-1);
		}

		return true;
	};
	unsigned long m_nextSN[3];
	bool m_isDone[3];
};

//
// Sender
//
class EventRbSender : public PafeeTask
{
public:
	EventRbSender(int senderID = 0, Listener* pListener=NULL)
	{
		m_senderID = senderID;
		m_pListener = pListener;
	};

protected:

	virtual void* TaskProcedure(void* pParam);

	int m_senderID;
	Listener* m_pListener;
};

void* EventRbSender::TaskProcedure(void* pParam)
{
	char szParam[256];
	unsigned long eventID;

	syslog(LOG_EMERG,"sender#%d start", m_senderID);
	for(int i = 0; i < 100; i++)
	{
		eventID = (m_senderID << 16) | i;
		sprintf(szParam, "event param#%lX", eventID);
		m_pListener->SendEvent(eventID, szParam, strlen(szParam));
		if((i % 10) == 0)
		{
			tslp_tsk(100);
		}
	}
	// Send end mark
	m_pListener->SendEvent((m_senderID << 16) | 0xEEEE, NULL, 0);
	syslog(LOG_EMERG,"sender#%d done", m_senderID);
	return NULL;
};


//
// Test main
//
void ringbufferedListenerTest(void)
{
	MyRingBufferedListener listener;
	EventRbSender sender1(1, &listener);
	EventRbSender sender2(2, &listener);
	EventRbSender sender3(3, &listener);

	//---------------------------------------------------------------
	// Three tasks send 1000 events to the listener.
	// the listener check the serial number of event and count it.
	//---------------------------------------------------------------

	listener.Initialize(3000, 256, TASK_LISTENER, EVTFLG_SIGNAL, SEM_MTX1);
	tslp_tsk(1000);

	sender1.Attach(TASK_SENDER1);
	sender1.Start();

	sender2.Attach(TASK_SENDER2);
	sender2.Start();

	sender3.Attach(TASK_SENDER3);
	sender3.Start();


	while(1)
	{
		if(listener.IsDone())
		{
			listener.ShowResult();
			break;
		}
		tslp_tsk(1000);
	}
	listener.Terminate();
	syslog(LOG_EMERG,"done,");
}

