#include "stdafx.h"
#include <time.h>
#include <new>

#include "PafeeStdAllocator.h"
#include "PafeeStdStreamOutput.h"
#include "PafeeFastFixedAllocator.h"
#include "PafeeFlexibleAllocator.h"
#include "PafeeLockedAllocator.h"

//#define _ALLOCATED_DATA_CHECK_

using namespace Pafee;

inline unsigned long GetUsec()
{
	return GetTickCount();
}

#define LOOP_COUNT		10
#define MAX_COUNT		1000

typedef struct _MEM_TEST
{
	unsigned long	size;			// allocation size
	unsigned long	timeNew;		// allocation time in usec
	unsigned long	timeDelete;		// deallocation  time in usec
	unsigned long	timeNewDelete;	// allocation and deallocation time in usec
} MEM_TEST;


MEM_TEST g_test[] ={{32,0,0,0},
					{64,0,0,0},
					{128,0,0,0},
					{256,0,0,0},
					{512,0,0,0},
					{1024,0,0,0},
					{2048,0,0,0},
					{4096,0,0,0},
					{8192,0,0,0},
					{16384,0,0,0},
					{32786,0,0,0},
					{65536,0,0,0},
					{0xffffffff,0,0,0}};
MEM_TEST g_randTest;

int g_rand[MAX_COUNT];
StdStreamOutput g_stream;


//
// Proto types
//
void testNewDelete(int count);
void testMallocFree(int count);
void testAllocator(int count, Allocator& alloc);

//---------------------------------------------
//	main 
//---------------------------------------------
int speedTest(void)
{
	int count = 100;
	int loop = 10;
	int i;
	

	srand(GetUsec());
	for(i = 0; i < count; i++)
	{
//		g_rand[i] = rand() & 0x3ffff;
		g_rand[i] = rand() & 0xffff;
	}

	printf("test started, loop count is %d,\n", count);
	printf("Test result details are... (time unit: usec)\n");
	printf("1st row: allocation time\n");
	printf("2nd row: deallocation time\n");
	printf("3rd row: allocation and deallocation time\n");
	printf("4th row: random allocation time\n");
	printf("5th row: random deallocation time\n");
	printf("6th row: random allocation and deallocation time\n");

	printf("\n");
	printf("Fixed allocation size are...\n");
	for(i = 0; g_test[i].size != 0xffffffff; i++)
	{
		printf("%ld,", g_test[i].size);
	}
	printf("\n");
	printf("Random allocation size are...\n");
	for(i = 0; i < count; i++)
	{
		printf("%d,", g_rand[i]);
	}
	printf("\n");

	try
	{

		printf("\n==== new & delete ==== \n");
		for(i = 0; i < loop; i++)
		{
			printf("round#%d\n", i);
			testNewDelete(count);
		}
		
		Sleep(1);
		printf("\n==== malloc & free ==== \n");
		for(i = 0; i < loop; i++)
		{
			printf("round#%d\n", i);
			testMallocFree(count);
		}
		
		Sleep(1);
		printf("\n==== StdAllocator ==== \n");
		for(i = 0; i < loop; i++)
		{
			printf("round#%d\n", i);
			StdAllocator alloc;
			testAllocator(count, alloc);
		}
		
		Sleep(1);
		printf("\n==== FlexibleAllocator ==== \n");
		for(i = 0; i < loop; i++)
		{
			printf("round#%d\n", i);
			FlexibleAllocator alloc;
			char* p = new char[20000000];
			alloc.Initialize((unsigned long) p, 20000000L, 20000000L, 64L);
			testAllocator(count, alloc);
			delete p;
		}

		Sleep(1);
		printf("\n==== FlexibleAllocatorWithLock ==== \n");
		for(i = 0; i < loop; i++)
		{
			printf("round#%d\n", i);
			FlexibleAllocator alloc;
			LockedAllocator   lockedAlloc;
			char* p = new char[20000000];
			alloc.Initialize((unsigned long) p, 20000000L, 20000000L, 64L);
			lockedAlloc.Initialize(&alloc);
			testAllocator(count, lockedAlloc);
			delete p;
		}
		
		Sleep(1);
		printf("\n==== FastFixedAllocator ==== \n");
		for(i = 0; i < loop; i++)
		{
			printf("round#%d\n", i);
			FastFixedAllocator alloc;
			alloc.Initialize(65536L+256L, 100L, NULL);
			testAllocator(count, alloc);
		}

		Sleep(1);
		printf("\n==== FastFixedAllocatorWithLock ==== \n");
		for(i = 0; i < loop; i++)
		{
			printf("round#%d\n", i);
			FastFixedAllocator alloc;
			alloc.Initialize(65536L+256L, 100L, NULL);
			LockedAllocator   lockedAlloc;
			lockedAlloc.Initialize(&alloc);
			testAllocator(count, lockedAlloc);
		}

		
	}
	catch(std::bad_alloc)
	{
		printf("xxx bad_alloc\n");
	}
	catch(...)
	{
		printf("xxx other exception\n");
	}
	return 0;
}


//---------------------------------------------
//	Show results
//---------------------------------------------
void showResults(void)
{
	int index;
	for(index = 0; g_test[index].size != 0xffffffff; index++)
	{
		printf("%ld,", g_test[index].timeNew);
	}
	printf("\n");
	for(index = 0; g_test[index].size != 0xffffffff; index++)
	{
		printf("%ld,", g_test[index].timeDelete);
	}
	printf("\n");
	for(index = 0; g_test[index].size != 0xffffffff; index++)
	{
		printf("%ld,", g_test[index].timeNewDelete);
	}
	printf("\n");
	printf("%ld,", g_randTest.timeNew);
	printf("\n");
	printf("%ld,", g_randTest.timeDelete);
	printf("\n");
	printf("%ld,", g_randTest.timeNewDelete);
	
	printf("\n");
}



//---------------------------------------------
//	new & delete test
//---------------------------------------------
void testNewDelete(int count)
{
	char** pAllocated = new char*[count];
	unsigned long startTime, endTime;
	int i, index;
	for(index = 0; g_test[index].size != 0xffffffff; index++)
	{
		// Allocation
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			pAllocated[i] = new char[g_test[index].size];
		}
		endTime = GetUsec();
		g_test[index].timeNew = endTime - startTime;
		
		// Deallocation
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			delete [] pAllocated[i];
		}
		endTime = GetUsec();
		g_test[index].timeDelete = endTime - startTime;
		
		// Allocation and Deallocation
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			pAllocated[i] = new char[g_test[index].size];
			delete [] pAllocated[i];
		}
		endTime = GetUsec();
		g_test[index].timeNewDelete = endTime - startTime;
		
	}
	// randam size alloc
	{
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			pAllocated[i] = new char[g_rand[i]];
		}
		endTime = GetUsec();
		g_randTest.timeNew = endTime - startTime;
	}

	// randam size free
	{
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			delete [] pAllocated[i];
		}
		endTime = GetUsec();
		g_randTest.timeDelete = endTime - startTime;
	}


	// randam size alloc & free
	{
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			pAllocated[i] = new char[g_rand[i]];
			delete [] pAllocated[i];
		}
		endTime = GetUsec();
		g_randTest.timeNewDelete = endTime - startTime;
	}

	
	showResults();
	delete [] pAllocated;
}

//---------------------------------------------
//	malloc & free test
//---------------------------------------------
void testMallocFree(int count)
{
	char** pAllocated = new char*[count];

	unsigned long startTime, endTime;
	int i, index;
	for(index = 0; g_test[index].size != 0xffffffff; index++)
	{
		// Allocation
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			pAllocated[i] = (char*) malloc(g_test[index].size);
		}
		endTime = GetUsec();
		g_test[index].timeNew = endTime - startTime;
		
		// Deallocation
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			free(pAllocated[i]);
		}
		endTime = GetUsec();
		g_test[index].timeDelete = endTime - startTime;
		
		// Allocation and Deallocation
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			pAllocated[i] =(char*)  malloc(g_test[index].size);
			free(pAllocated[i]);
		}
		endTime = GetUsec();
		g_test[index].timeNewDelete = endTime - startTime;
	}
	// randam size alloc
	{
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			pAllocated[i] = (char*)malloc(g_rand[i]);
		}
		endTime = GetUsec();
		g_randTest.timeNew = endTime - startTime;
	}

	// randam size free
	{
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			free(pAllocated[i]);
		}
		endTime = GetUsec();
		g_randTest.timeDelete = endTime - startTime;
	}


	// randam size alloc & free
	{
		startTime = GetUsec();
		for(i = 0; i < count; i++)
		{
			pAllocated[i] = (char*)malloc(g_rand[i]);
			free(pAllocated[i]);
		}
		endTime = GetUsec();
		g_randTest.timeNewDelete = endTime - startTime;
	}

	showResults();
	delete [] pAllocated;
}

//---------------------------------------------
//	Allocator test
//---------------------------------------------
void testAllocator(int count, Allocator& alloc)
{
	char** pAllocated = new char*[count];
	
	try
	{
		unsigned long startTime, endTime;
		int i,index;
		for(index = 0; g_test[index].size != 0xffffffff; index++)
		{
			// Allocation
			startTime = GetUsec();
			for(i = 0; i < count; i++)
			{
				pAllocated[i] = (char*) alloc.Allocate(g_test[index].size);
				if(pAllocated[i] == NULL)
				{
					abort();
				}

#ifdef _ALLOCATED_DATA_CHECK_
				memset(pAllocated[i], 0xcd, g_test[index].size);
#endif
			}
			endTime = GetUsec();
			g_test[index].timeNew = endTime - startTime;
			// Deallocation
			startTime = GetUsec();
			for(i = 0; i < count; i++)
			{
				alloc.Deallocate(pAllocated[i]);
			}
			endTime = GetUsec();
			g_test[index].timeDelete = endTime - startTime;
			
			// Allocation and Deallocation
			startTime = GetUsec();
			for(i = 0; i < count; i++)
			{
				pAllocated[i] =(char*) alloc.Allocate(g_test[index].size);
				if(pAllocated[i] == NULL)
				{
					abort();
				}
#ifdef _ALLOCATED_DATA_CHECK_
				memset(pAllocated[i], 0xcd, g_test[index].size);
#endif
				alloc.Deallocate(pAllocated[i]);
			}
			endTime = GetUsec();
			g_test[index].timeNewDelete = endTime - startTime;
		}
		// randam size alloc
		{
			startTime = GetUsec();
			for(i = 0; i < count; i++)
			{
				pAllocated[i] = (char*)alloc.Allocate(g_rand[i]);
				if(pAllocated[i] == NULL)
				{
					abort();
				}
#ifdef _ALLOCATED_DATA_CHECK_
				memset(pAllocated[i], 0xcd, g_rand[i]);
#endif
			}
			endTime = GetUsec();
			g_randTest.timeNew = endTime - startTime;
		}
		// randam size free
		{
			startTime = GetUsec();
			for(i = 0; i < count; i++)
			{
				alloc.Deallocate(pAllocated[i]);
			}
			endTime = GetUsec();
			g_randTest.timeDelete = endTime - startTime;
		}

		// randam size alloc & free
		{
			startTime = GetUsec();
			for(i = 0; i < count; i++)
			{
				pAllocated[i] = (char*)alloc.Allocate(g_rand[i]);
				if(pAllocated[i] == NULL)
				{
					abort();
				}
#ifdef _ALLOCATED_DATA_CHECK_
				memset(pAllocated[i], 0xcd, g_rand[i]);
#endif
				alloc.Deallocate(pAllocated[i]);
			}
			endTime = GetUsec();
			g_randTest.timeNewDelete = endTime - startTime;
		}
		showResults();
	}
	catch(std::bad_alloc e)
	{
		printf("XXXX bad alloc %d %s\n", __LINE__, e.what());
		abort();
	}
	catch(std::exception e)
	{
		printf("XXXX exception %d %s\n", __LINE__, e.what());
		abort();
	}
	catch(...)
	{
		printf("XXXX other exception %d\n", __LINE__);
		abort();
	}
	delete [] pAllocated;
}

